@@ -2,10 +2,11 @@
 # License: GPL v3 Copyright: 2018, Kovid Goyal <kovid at kovidgoyal.net>
 
 import os
+import re
 import sys
 from contextlib import suppress
 from typing import (
-    TYPE_CHECKING, Callable, Dict, List, NamedTuple, Optional, Tuple
+    TYPE_CHECKING, Callable, Dict, Iterator, List, NamedTuple, Optional, Tuple
 )
 
 from kitty.cli import parse_args
@@ -16,11 +17,12 @@ from kitty.typing import BossType, KeyEventType, TypedDict
 from kitty.utils import ScreenSize
 
 from ..tui.handler import Handler, result_handler
-from ..tui.loop import Loop, MouseEvent
+from ..tui.loop import Loop, MouseEvent, debug
 from ..tui.operations import MouseTracking, alternate_screen, styled
 
 if TYPE_CHECKING:
     import readline
+    debug
 else:
     readline = None
 
@@ -108,6 +110,15 @@ The default choice is selected when the user presses the :kbd:`Enter` key.
 --prompt -p
 default="> "
 The prompt to use when inputting a line of text or a password.
+
+
+--unhide-key
+default=u
+The key to be pressed to unhide hidden text
+
+
+--hidden-text-placeholder
+The text in the message to be replaced by hidden text. The hidden text is read via STDIN.
 '''
 
 
@@ -198,6 +209,7 @@ class Choose(Handler):
     mouse_tracking = MouseTracking.buttons_only
 
     def __init__(self, cli_opts: AskCLIOptions) -> None:
+        self.prefix_style_pat = re.compile(r'(?:\x1b\[[^m]*?m)+')
         self.cli_opts = cli_opts
         self.choices: Dict[str, Choice] = {}
         self.clickable_ranges: Dict[str, List[Range]] = {}
@@ -219,6 +231,18 @@ class Choose(Handler):
         self.response_on_accept = cli_opts.default or ''
         if cli_opts.type in ('yesno', 'choices') and self.response_on_accept not in self.allowed:
             self.response_on_accept = 'y' if cli_opts.type == 'yesno' else tuple(self.choices.keys())[0]
+        self.message = cli_opts.message
+        self.hidden_text_start_pos = self.hidden_text_end_pos = -1
+        self.hidden_text = ''
+        self.replacement_text = t = f'Press {styled(self.cli_opts.unhide_key, fg="green")} or click to show'
+        self.replacement_range = Range(-1, -1, -1)
+        if self.message and self.cli_opts.hidden_text_placeholder:
+            self.hidden_text_start_pos = self.message.find(self.cli_opts.hidden_text_placeholder)
+            if self.hidden_text_start_pos > -1:
+                self.hidden_text = sys.stdin.read().rstrip()
+                self.hidden_text_end_pos = self.hidden_text_start_pos + len(t)
+                suffix = self.message[self.hidden_text_start_pos + len(self.cli_opts.hidden_text_placeholder):]
+                self.message = self.message[:self.hidden_text_start_pos] + t + suffix
 
     def initialize(self) -> None:
         self.cmd.set_cursor_visible(False)
@@ -227,24 +251,35 @@ class Choose(Handler):
     def finalize(self) -> None:
         self.cmd.set_cursor_visible(True)
 
-    def draw_long_text(self, text: str) -> int:
-        y = 0
+    def draw_long_text(self, text: str) -> Iterator[str]:
+        if not text:
+            yield ''
+            return
         width = self.screen_size.cols - 2
+        m = self.prefix_style_pat.match(text)
+        prefix = m.group() if m else ''
         while text:
             t, text = truncate_at_space(text, width)
             t = t.strip()
-            self.print(' ' * extra_for(wcswidth(t), width), styled(t, bold=True), sep='')
-            y += 1
-        return y
+            yield ' ' * extra_for(wcswidth(t), width) + styled(prefix + t, bold=True)
 
     @Handler.atomic_update
     def draw_screen(self) -> None:
         self.cmd.clear_screen()
-        y = max(0, self.screen_size.rows // 2 - 2)
+        msg_lines: List[str] = []
+        if self.message:
+            for line in self.message.splitlines():
+                msg_lines.extend(self.draw_long_text(line))
+        y = self.screen_size.rows - len(msg_lines)
+        y = max(0, (y // 2) - 2)
         self.print(end='\r\n'*y)
-        if self.cli_opts.message:
-            for line in self.cli_opts.message.splitlines():
-                y += self.draw_long_text(line)
+        for line in msg_lines:
+            if self.replacement_text in line:
+                idx = line.find(self.replacement_text)
+                x = wcswidth(line[:idx])
+                self.replacement_range = Range(x, x + wcswidth(self.replacement_text), y)
+            self.print(line)
+            y += 1
         if self.screen_size.rows > 2:
             self.print()
             y += 1
@@ -259,7 +294,8 @@ class Choose(Handler):
         current_line_length = 0
         current_line: List[Tuple[str, str]] = []
         lines: List[List[Tuple[str, str]]] = []
-        sep, sep_sz = '  ', 2
+        sep = '  '
+        sep_sz = len(sep) + 2  # for the borders
 
         for choice in choices:
             self.clickable_ranges[choice.letter] = []
@@ -272,7 +308,7 @@ class Choose(Handler):
                 current_line = []
                 current_line_length = 0
             current_line.append((choice.letter, text))
-            current_line_length += sz
+            current_line_length += sz + sep_sz
         if current_line:
             lines.append(current_line)
 
@@ -311,10 +347,12 @@ class Choose(Handler):
             self.print(' ' * offset, line, sep='', end='' if is_last else '\r\n')
             y += 1
 
+        self.cmd.set_line_wrapping(False)
         for boxed_line in lines:
             print_line(top, *boxed_line)
             print_line(middle, *boxed_line)
             print_line(bottom, *boxed_line, is_last=boxed_line is lines[-1])
+        self.cmd.set_line_wrapping(True)
 
     def draw_choice(self, y: int) -> None:
         if y + 3 <= self.screen_size.rows:
@@ -370,6 +408,14 @@ class Choose(Handler):
             self.quit_loop(0)
         elif self.cli_opts.type == 'yesno':
             self.on_interrupt()
+        elif self.hidden_text and text == self.cli_opts.unhide_key:
+            self.unhide()
+
+    def unhide(self) -> None:
+        if self.hidden_text and self.message:
+            self.message = self.message[:self.hidden_text_start_pos] + self.hidden_text + self.message[self.hidden_text_end_pos:]
+            self.hidden_text = ''
+            self.draw_screen()
 
     def on_key(self, key_event: KeyEventType) -> None:
         if key_event.matches('esc'):
@@ -385,6 +431,8 @@ class Choose(Handler):
                     self.response = letter
                     self.quit_loop(0)
                     return
+        if self.hidden_text and self.replacement_range.has_point(ev.cell_x, ev.cell_y):
+            self.unhide()
 
     def on_resize(self, screen_size: ScreenSize) -> None:
         self.screen_size = screen_size
