@@ -27,7 +27,6 @@ from typing import (
     Tuple, Union, cast
 )
 
-from kittens.tui.operations import restore_colors, save_colors
 from kitty.constants import (
     cache_dir, runtime_dir, shell_integration_dir, ssh_control_master_template,
     str_version, terminfo_dir
@@ -40,7 +39,11 @@ from kitty.utils import (
     set_echo as turn_off_echo
 )
 
-from ..tui.utils import kitty_opts
+from ..tui.operations import (
+    RESTORE_PRIVATE_MODE_VALUES, SAVE_PRIVATE_MODE_VALUES, Mode,
+    restore_colors, save_colors, set_mode
+)
+from ..tui.utils import kitty_opts, running_in_tmux
 from .config import init_config
 from .copy import CopyInstruction
 from .options.types import Options as SSHOptions
@@ -168,6 +171,8 @@ def make_tarfile(ssh_opts: SSHOptions, base_env: Dict[str, str], compression: st
         env['KITTY_LOGIN_CWD'] = ssh_opts.cwd
     if ssh_opts.remote_kitty != 'no':
         env['KITTY_REMOTE'] = ssh_opts.remote_kitty
+    if os.environ.get('KITTY_PUBLIC_KEY'):
+        env['KITTY_PUBLIC_KEY'] = os.environ['KITTY_PUBLIC_KEY']
     env_script = serialize_env(literal_env, env, base_env, for_python=compression != 'gz')
     buf = io.BytesIO()
     with tarfile.open(mode=f'w:{compression}', fileobj=buf, encoding='utf-8') as tf:
@@ -209,7 +214,7 @@ def get_ssh_data(msg: str, request_id: str) -> Iterator[bytes]:
             if pw != env_data['pw']:
                 raise ValueError('Incorrect password')
             if rq_id != request_id:
-                raise ValueError('Incorrect request id')
+                raise ValueError(f'Incorrect request id: {rq_id!r} expecting the KITTY_PID-KITTY_WINDOW_ID for the current kitty window')
         except Exception as e:
             traceback.print_exc()
             yield f'{e}\n'.encode('utf-8')
@@ -409,9 +414,11 @@ class InvalidSSHArgs(ValueError):
         os.execlp(ssh_exe(), 'ssh')
 
 
+passthrough_args = {f'-{x}' for x in 'NnfGT'}
+
+
 def parse_ssh_args(args: List[str], extra_args: Tuple[str, ...] = ()) -> Tuple[List[str], List[str], bool, Tuple[str, ...]]:
     boolean_ssh_args, other_ssh_args = get_ssh_cli()
-    passthrough_args = {f'-{x}' for x in 'NnfG'}
     ssh_args = []
     server_args: List[str] = []
     expecting_option_val = False
@@ -508,7 +515,7 @@ def get_remote_command(
     return wrap_bootstrap_script(sh_script, interpreter), replacements, shm_name
 
 
-def connection_sharing_args(opts: SSHOptions, kitty_pid: int) -> List[str]:
+def connection_sharing_args(kitty_pid: int) -> List[str]:
     rd = runtime_dir()
     # Bloody OpenSSH generates a 40 char hash and in creating the socket
     # appends a 27 char temp suffix to it. Socket max path length is approx
@@ -547,17 +554,29 @@ def connection_sharing_args(opts: SSHOptions, kitty_pid: int) -> List[str]:
 def restore_terminal_state() -> Iterator[bool]:
     with open(os.ctermid()) as f:
         val = termios.tcgetattr(f.fileno())
+        print(end=SAVE_PRIVATE_MODE_VALUES)
+        print(end=set_mode(Mode.HANDLE_TERMIOS_SIGNALS), flush=True)
         try:
             yield bool(val[3] & termios.ECHO)
         finally:
             termios.tcsetattr(f.fileno(), termios.TCSAFLUSH, val)
+            print(end=RESTORE_PRIVATE_MODE_VALUES, flush=True)
 
 
 def dcs_to_kitty(payload: Union[bytes, str], type: str = 'ssh') -> bytes:
     if isinstance(payload, str):
         payload = payload.encode('utf-8')
     payload = standard_b64encode(payload)
-    return b'\033P@kitty-' + type.encode('ascii') + b'|' + payload + b'\033\\'
+    ans = b'\033P@kitty-' + type.encode('ascii') + b'|' + payload
+    tmux = running_in_tmux()
+    if tmux:
+        cp = subprocess.run([tmux, 'set', '-p', 'allow-passthrough', 'on'])
+        if cp.returncode != 0:
+            raise SystemExit(cp.returncode)
+        ans = b'\033Ptmux;\033' + ans + b'\033\033\\\033\\'
+    else:
+        ans += b'\033\\'
+    return ans
 
 
 @run_once
@@ -571,7 +590,6 @@ def ssh_version() -> Tuple[int, int]:
 
 @contextmanager
 def drain_potential_tty_garbage(p: 'subprocess.Popen[bytes]', data_request: str) -> Iterator[None]:
-    ssh_started_at = time.monotonic()
     with open(os.open(os.ctermid(), os.O_CLOEXEC | os.O_RDWR | os.O_NOCTTY), 'wb') as tty:
         if data_request:
             turn_off_echo(tty.fileno())
@@ -580,14 +598,18 @@ def drain_potential_tty_garbage(p: 'subprocess.Popen[bytes]', data_request: str)
         try:
             yield
         finally:
-            if p.returncode and time.monotonic() - ssh_started_at < 30:
-                # discard queued input data on tty in case data transmission was
-                # interrupted due to SSH failure, avoids spewing garbage to
-                # screen
-                data = b''
-                give_up_at = time.monotonic() + 1
-                tty_fd = tty.fileno()
-                while time.monotonic() < give_up_at and b'KITTY_DATA_END' not in data:
+            # discard queued input data on tty in case data transmission was
+            # interrupted due to SSH failure, avoids spewing garbage to screen
+            from uuid import uuid4
+            canary = uuid4().hex.encode('ascii')
+            turn_off_echo(tty.fileno())
+            tty.write(dcs_to_kitty(canary + b'\n\r', type='echo'))
+            tty.flush()
+            data = b''
+            give_up_at = time.monotonic() + 2
+            tty_fd = tty.fileno()
+            while time.monotonic() < give_up_at and canary not in data:
+                with suppress(KeyboardInterrupt):
                     rd, wr, err = select([tty_fd], [], [tty_fd], max(0, give_up_at - time.monotonic()))
                     if err or not rd:
                         break
@@ -669,7 +691,7 @@ def run_ssh(ssh_args: List[str], server_args: List[str], found_extra_args: Tuple
         overrides.insert(0, f'hostname {uname}@{hostname_for_match}')
     host_opts = init_config(hostname_for_match, uname, overrides)
     if host_opts.share_connections:
-        cmd[insertion_point:insertion_point] = connection_sharing_args(host_opts, int(os.environ['KITTY_PID']))
+        cmd[insertion_point:insertion_point] = connection_sharing_args(int(os.environ['KITTY_PID']))
     use_kitty_askpass = host_opts.askpass == 'native' or (host_opts.askpass == 'unless-set' and 'SSH_ASKPASS' not in os.environ)
     need_to_request_data = True
     if use_kitty_askpass:
@@ -699,16 +721,13 @@ def run_ssh(ssh_args: List[str], server_args: List[str], found_extra_args: Tuple
         else:
             rq = '' if need_to_request_data else 'id={REQUEST_ID}:pwfile={PASSWORD_FILENAME}:pw={DATA_PASSWORD}'.format(**replacements)
             with drain_potential_tty_garbage(p, rq):
-                try:
-                    raise SystemExit(p.wait())
-                except KeyboardInterrupt:
-                    raise SystemExit(1)
+                raise SystemExit(p.wait())
         finally:
             if colors_changed:
                 print(end=restore_colors(), flush=True)
 
 
-def main(args: List[str]) -> NoReturn:
+def main(args: List[str]) -> None:
     args = args[1:]
     if args and args[0] == 'use-python':
         args = args[1:]  # backwards compat from when we had a python implementation
@@ -716,13 +735,20 @@ def main(args: List[str]) -> NoReturn:
         ssh_args, server_args, passthrough, found_extra_args = parse_ssh_args(args, extra_args=('--kitten',))
     except InvalidSSHArgs as e:
         e.system_exit()
+    if passthrough:
+        if found_extra_args:
+            raise SystemExit(f'The SSH kitten cannot work with the options: {", ".join(passthrough_args)}')
+        os.execlp(ssh_exe(), 'ssh', *args)
+
     if not os.environ.get('KITTY_WINDOW_ID') or not os.environ.get('KITTY_PID'):
         raise SystemExit('The SSH kitten is meant to run inside a kitty window')
-    if passthrough:
-        raise SystemExit('The SSH kitten is meant for interactive use via SSH only')
     if not sys.stdin.isatty():
         raise SystemExit('The SSH kitten is meant for interactive use only, STDIN must be a terminal')
-    run_ssh(ssh_args, server_args, found_extra_args)
+    try:
+        run_ssh(ssh_args, server_args, found_extra_args)
+    except KeyboardInterrupt:
+        sys.excepthook = lambda *a: None
+        raise
 
 
 if __name__ == '__main__':
