@@ -20,6 +20,7 @@
 #include <sys/ioctl.h>
 #include <sys/wait.h>
 #include <sys/socket.h>
+#include <sys/un.h>
 #include <signal.h>
 extern PyTypeObject Screen_Type;
 
@@ -33,7 +34,7 @@ extern PyTypeObject Screen_Type;
 #define EVDBG(...)
 #endif
 
-#define EXTRA_FDS 2
+#define EXTRA_FDS 3
 #ifndef MSG_NOSIGNAL
 // Apple does not implement MSG_NOSIGNAL
 #define MSG_NOSIGNAL 0
@@ -56,7 +57,7 @@ typedef struct {
     bool shutting_down;
     pthread_t io_thread, talk_thread;
 
-    int talk_fd, listen_fd;
+    int talk_fd, listen_fd, prewarm_fd;
     Message *messages;
     size_t messages_capacity, messages_count;
     LoopData io_loop_data;
@@ -123,11 +124,11 @@ static PyObject *
 new(PyTypeObject *type, PyObject *args, PyObject UNUSED *kwds) {
     ChildMonitor *self;
     PyObject *dump_callback, *death_notify;
-    int talk_fd = -1, listen_fd = -1;
+    int talk_fd = -1, listen_fd = -1, prewarm_fd = -1;
     int ret;
 
     if (the_monitor) { PyErr_SetString(PyExc_RuntimeError, "Can have only a single ChildMonitor instance"); return NULL; }
-    if (!PyArg_ParseTuple(args, "OO|ii", &death_notify, &dump_callback, &talk_fd, &listen_fd)) return NULL;
+    if (!PyArg_ParseTuple(args, "OO|iii", &death_notify, &dump_callback, &talk_fd, &listen_fd, &prewarm_fd)) return NULL;
     if ((ret = pthread_mutex_init(&children_lock, NULL)) != 0) {
         PyErr_Format(PyExc_RuntimeError, "Failed to create children_lock mutex: %s", strerror(ret));
         return NULL;
@@ -137,9 +138,10 @@ new(PyTypeObject *type, PyObject *args, PyObject UNUSED *kwds) {
         return NULL;
     }
     self = (ChildMonitor *)type->tp_alloc(type, 0);
-    if (!init_loop_data(&self->io_loop_data, 5, SIGINT, SIGTERM, SIGCHLD, SIGUSR1, SIGUSR2)) return PyErr_SetFromErrno(PyExc_OSError);
+    if (!init_loop_data(&self->io_loop_data, SIGINT, SIGHUP, SIGTERM, SIGCHLD, SIGUSR1, SIGUSR2, 0)) return PyErr_SetFromErrno(PyExc_OSError);
     self->talk_fd = talk_fd;
     self->listen_fd = listen_fd;
+    self->prewarm_fd = prewarm_fd;
     if (self == NULL) return PyErr_NoMemory();
     self->death_notify = death_notify; Py_INCREF(death_notify);
     if (dump_callback != Py_None) {
@@ -148,7 +150,8 @@ new(PyTypeObject *type, PyObject *args, PyObject UNUSED *kwds) {
     } else parse_func = parse_worker;
     self->count = 0;
     children_fds[0].fd = self->io_loop_data.wakeup_read_fd; children_fds[1].fd = self->io_loop_data.signal_read_fd;
-    children_fds[0].events = POLLIN; children_fds[1].events = POLLIN;
+    children_fds[2].fd = self->prewarm_fd;
+    children_fds[0].events = POLLIN; children_fds[1].events = POLLIN; children_fds[2].events = POLLIN;
     the_monitor = self;
 
     return (PyObject*) self;
@@ -174,9 +177,21 @@ dealloc(ChildMonitor* self) {
         FREE_CHILD(add_queue[add_queue_count]);
     }
     free_loop_data(&self->io_loop_data);
+    safe_close(self->prewarm_fd, __FILE__, __LINE__); self->prewarm_fd = -1;
     Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
+static PyObject*
+handled_signals(ChildMonitor *self, PyObject *args UNUSED) {
+    PyObject *ans = PyTuple_New(self->io_loop_data.num_handled_signals);
+    if (ans) {
+        for (Py_ssize_t i = 0; i < PyTuple_GET_SIZE(ans); i++) {
+            PyTuple_SET_ITEM(ans, i, PyLong_FromLong((long)self->io_loop_data.handled_signals[i]));
+        }
+    }
+    return ans;
+}
+
 static void
 wakeup_io_loop(ChildMonitor *self, bool in_signal_handler) {
     wakeup_loop(&self->io_loop_data, in_signal_handler, "io_loop");
@@ -427,7 +442,7 @@ parse_input(ChildMonitor *self) {
             }
             if (resp) {
                 if (PyBytes_Check(resp)) send_response_to_peer(msg->peer_id, PyBytes_AS_STRING(resp), PyBytes_GET_SIZE(resp));
-                else if (resp == Py_None || resp == Py_True) send_response_to_peer(msg->peer_id, NULL, 0);
+                else if (resp == Py_None) send_response_to_peer(msg->peer_id, NULL, 0);
                 Py_CLEAR(resp);
             } else send_response_to_peer(msg->peer_id, NULL, 0);
         }
@@ -743,7 +758,6 @@ render(monotonic_t now, bool input_read) {
 
     for (size_t i = 0; i < global_state.num_os_windows; i++) {
         OSWindow *w = global_state.os_windows + i;
-        w->render_calls++;
         if (!w->num_tabs) continue;
         if (!should_os_window_be_rendered(w)) {
             update_os_window_title(w);
@@ -752,8 +766,11 @@ render(monotonic_t now, bool input_read) {
         }
         if (USE_RENDER_FRAMES && w->render_state != RENDER_FRAME_READY) {
             if (w->render_state == RENDER_FRAME_NOT_REQUESTED || no_render_frame_received_recently(w, now, ms_to_monotonic_t(250ll))) request_frame_render(w);
-            continue;
+            // dont respect render frames soon after a resize on Wayland as they cause flicker because
+            // we want to fill the newly resized buffer ASAP, not at compositors convenience
+            if (global_state.is_wayland && (monotonic() - w->viewport_resized_at) > s_double_to_monotonic_t(1)) continue;
         }
+        w->render_calls++;
         make_os_window_context_current(w);
         if (w->live_resize.in_progress && OPT(resize_draw_strategy) >= RESIZE_DRAW_BLANK) {
             blank_os_window(w);
@@ -1240,12 +1257,13 @@ read_bytes(int fd, Screen *screen) {
 
 typedef struct { bool kill_signal, child_died, reload_config; } SignalSet;
 
-static void
+static bool
 handle_signal(const siginfo_t *siginfo, void *data) {
     SignalSet *ss = data;
     switch(siginfo->si_signo) {
         case SIGINT:
         case SIGTERM:
+        case SIGHUP:
             ss->kill_signal = true;
             break;
         case SIGCHLD:
@@ -1260,6 +1278,7 @@ handle_signal(const siginfo_t *siginfo, void *data) {
         default:
             break;
     }
+    return true;
 }
 
 static void
@@ -1289,6 +1308,34 @@ mark_monitored_pids(pid_t pid, int status) {
     children_mutex(unlock);
 }
 
+static void
+reap_prewarmed_children(ChildMonitor *self, int fd, bool enable_close_on_child_death) {
+    static char buf[256];
+    static size_t buf_pos = 0;
+    while(true) {
+        ssize_t len = read(fd, buf + buf_pos, sizeof(buf) - buf_pos);
+        if (len < 0) {
+            if (errno == EINTR) continue;
+            if (errno != EIO && errno != EAGAIN) log_error("Call to read() from reap_prewarmed_children() failed with error: %s", strerror(errno));
+            break;
+        }
+        buf_pos += len;
+        char *nl;
+        while (buf_pos > 1 && (nl = memchr(buf, '\n', buf_pos)) != NULL) {
+            size_t sz = nl - buf + 1;
+            if (enable_close_on_child_death) {
+                *nl = 0;
+                int pid = atoi(buf);
+                if (pid) mark_child_for_removal(self, pid);
+            }
+            memmove(buf, buf + sz, sz);
+            buf_pos -= sz;
+        }
+        if (len == 0) break;
+    }
+
+}
+
 static void
 reap_children(ChildMonitor *self, bool enable_close_on_child_death) {
     int status;
@@ -1379,6 +1426,9 @@ io_loop(void *data) {
                 }
                 if (ss.child_died) reap_children(self, OPT(close_on_child_death));
             }
+            if (children_fds[2].revents && POLLIN) {
+                reap_prewarmed_children(self, children_fds[2].fd, OPT(close_on_child_death));
+            }
             for (i = 0; i < self->count; i++) {
                 if (children_fds[EXTRA_FDS + i].revents & (POLLIN | POLLHUP)) {
                     data_received = true;
@@ -1682,8 +1732,10 @@ send_response_to_peer(id_type peer_id, const char *msg, size_t msg_sz) {
                         peer->write.capacity += msg_sz;
                     } else fatal("Out of memory");
                 }
-                if (msg) memcpy(peer->write.data + peer->write.used, msg, msg_sz);
-                peer->write.used += msg_sz;
+                if (msg_sz && msg) {
+                    memcpy(peer->write.data + peer->write.used, msg, msg_sz);
+                    peer->write.used += msg_sz;
+                }
             }
             wakeup = true;
             break;
@@ -1705,6 +1757,7 @@ static PyMethodDef methods[] = {
     METHOD(main_loop, METH_NOARGS)
     METHOD(mark_for_close, METH_VARARGS)
     METHOD(resize_pty, METH_VARARGS)
+    METHODB(handled_signals, METH_NOARGS),
     {"set_iutf8_winid", (PyCFunction)pyset_iutf8, METH_VARARGS, ""},
     {NULL}  /* Sentinel */
 };
@@ -1743,7 +1796,6 @@ cocoa_set_menubar_title(PyObject *self UNUSED, PyObject *args UNUSED) {
 }
 
 static PyObject*
-
 send_data_to_peer(PyObject *self UNUSED, PyObject *args) {
     char * msg; Py_ssize_t sz;
     unsigned long long peer_id;
@@ -1752,8 +1804,28 @@ send_data_to_peer(PyObject *self UNUSED, PyObject *args) {
     Py_RETURN_NONE;
 }
 
+static PyObject *
+random_unix_socket(PyObject *self UNUSED, PyObject *args UNUSED) {
+#ifndef SO_PASSCRED
+    errno = ENOTSUP;
+    return PyErr_SetFromErrno(PyExc_OSError);
+#else
+    int fd, optval = 1;
+    struct sockaddr_un bind_addr = {.sun_family=AF_UNIX};
+    fd = socket(AF_UNIX, SOCK_STREAM, 0);
+    if (fd < 0) return PyErr_SetFromErrno(PyExc_OSError);
+    if (setsockopt(fd, SOL_SOCKET, SO_PASSCRED, &optval, sizeof optval) != 0) goto fail;
+    if (bind(fd, (struct sockaddr *)&bind_addr, sizeof(sa_family_t)) != 0) goto fail;
+    return PyLong_FromLong((long)fd);
+fail:
+    safe_close(fd, __FILE__, __LINE__);
+    return PyErr_SetFromErrno(PyExc_OSError);
+#endif
+}
+
 static PyMethodDef module_methods[] = {
     METHODB(safe_pipe, METH_VARARGS),
+    METHODB(random_unix_socket, METH_NOARGS),
     {"add_timer", (PyCFunction)add_python_timer, METH_VARARGS, ""},
     {"remove_timer", (PyCFunction)remove_python_timer, METH_VARARGS, ""},
     METHODB(monitor_pid, METH_VARARGS),
