@@ -13,7 +13,8 @@ from typing import (
 import kitty.fast_data_types as fast_data_types
 
 from .constants import (
-    is_freebsd, is_macos, kitty_base_dir, shell_path, terminfo_dir
+    handled_signals, is_freebsd, is_macos, kitty_base_dir, shell_path,
+    terminfo_dir
 )
 from .types import run_once
 from .utils import log_error, which
@@ -142,14 +143,6 @@ def environ_of_process(pid: int) -> Dict[str, str]:
     return parse_environ_block(_environ_of_process(pid))
 
 
-def remove_cloexec(fd: int) -> None:
-    os.set_inheritable(fd, True)
-
-
-def remove_blocking(fd: int) -> None:
-    os.set_blocking(fd, False)
-
-
 def process_env() -> Dict[str, str]:
     ans = dict(os.environ)
     ssl_env_var = getattr(sys, 'kitty_ssl_env_var', None)
@@ -175,9 +168,14 @@ def set_default_env(val: Optional[Dict[str, str]] = None) -> None:
     setattr(default_env, 'lc_ctype_set_by_user', has_lctype)
 
 
+def set_LANG_in_default_env(val: str) -> None:
+    default_env()['LANG'] = val
+
+
 def openpty() -> Tuple[int, int]:
     master, slave = os.openpty()  # Note that master and slave are in blocking mode
-    remove_cloexec(slave)
+    os.set_inheritable(slave, True)
+    os.set_inheritable(master, False)
     fast_data_types.set_iutf8_fd(master, True)
     return master, slave
 
@@ -193,11 +191,33 @@ class ProcessDesc(TypedDict):
     cmdline: Optional[Sequence[str]]
 
 
+def is_prewarmable(argv: Sequence[str]) -> bool:
+    if len(argv) < 3 or os.path.basename(argv[0]) != 'kitty':
+        return False
+    if argv[1][:1] not in '@+':
+        return False
+    if argv[1][0] == '@':
+        return True
+    if argv[1] == '+':
+        return argv[2] != 'open'
+    return argv[1] != '+open'
+
+
+@run_once
+def cmdline_of_prewarmer() -> List[str]:
+    # we need this check in case the prewarmed process has done an exec and
+    # changed its cmdline
+    with suppress(Exception):
+        return cmdline_of_pid(fast_data_types.get_boss().prewarm.worker_pid)
+    return ['']
+
+
 class Child:
 
     child_fd: Optional[int] = None
     pid: Optional[int] = None
     forked = False
+    is_prewarmed = False
 
     def __init__(
         self,
@@ -206,11 +226,11 @@ class Child:
         stdin: Optional[bytes] = None,
         env: Optional[Dict[str, str]] = None,
         cwd_from: Optional['CwdRequest'] = None,
-        allow_remote_control: bool = False,
         is_clone_launch: str = '',
+        add_listen_on_env_var: bool = True,
     ):
-        self.allow_remote_control = allow_remote_control
         self.is_clone_launch = is_clone_launch
+        self.add_listen_on_env_var = add_listen_on_env_var
         self.argv = list(argv)
         if cwd_from:
             try:
@@ -226,13 +246,19 @@ class Child:
     def final_env(self) -> Dict[str, str]:
         from kitty.options.utils import DELETE_ENV_VAR
         env = default_env().copy()
-        if is_macos and env.get('LC_CTYPE') == 'UTF-8' and not sys._xoptions.get(
+        boss = fast_data_types.get_boss()
+        if is_macos and env.get('LC_CTYPE') == 'UTF-8' and not getattr(sys, 'kitty_run_data').get(
                 'lc_ctype_before_python') and not getattr(default_env, 'lc_ctype_set_by_user', False):
             del env['LC_CTYPE']
         env.update(self.env)
         env['TERM'] = fast_data_types.get_options().term
         env['COLORTERM'] = 'truecolor'
         env['KITTY_PID'] = getpid()
+        env['KITTY_PUBLIC_KEY'] = boss.encryption_public_key
+        if self.add_listen_on_env_var and boss.listening_on:
+            env['KITTY_LISTEN_ON'] = boss.listening_on
+        else:
+            env.pop('KITTY_LISTEN_ON', None)
         if self.cwd:
             # needed in case cwd is a symlink, in which case shells
             # can use it to display the current directory name rather
@@ -261,14 +287,18 @@ class Child:
         self.forked = True
         master, slave = openpty()
         stdin, self.stdin = self.stdin, None
-        ready_read_fd, ready_write_fd = os.pipe()
-        remove_cloexec(ready_read_fd)
-        if stdin is not None:
-            stdin_read_fd, stdin_write_fd = os.pipe()
-            remove_cloexec(stdin_read_fd)
-        else:
-            stdin_read_fd = stdin_write_fd = -1
-        env = tuple(f'{k}={v}' for k, v in self.final_env().items())
+        self.is_prewarmed = is_prewarmable(self.argv)
+        if not self.is_prewarmed:
+            ready_read_fd, ready_write_fd = os.pipe()
+            os.set_inheritable(ready_write_fd, False)
+            os.set_inheritable(ready_read_fd, True)
+            if stdin is not None:
+                stdin_read_fd, stdin_write_fd = os.pipe()
+                os.set_inheritable(stdin_write_fd, False)
+                os.set_inheritable(stdin_read_fd, True)
+            else:
+                stdin_read_fd = stdin_write_fd = -1
+            env = tuple(f'{k}={v}' for k, v in self.final_env().items())
         argv = list(self.argv)
         exe = argv[0]
         if is_macos and exe == shell_path:
@@ -289,29 +319,46 @@ class Child:
             argv[0] = (f'-{exe.split("/")[-1]}')
         self.final_exe = which(exe) or exe
         self.final_argv0 = argv[0]
-        pid = fast_data_types.spawn(self.final_exe, self.cwd, tuple(argv), env, master, slave, stdin_read_fd, stdin_write_fd, ready_read_fd, ready_write_fd)
+        if self.is_prewarmed:
+            fe = self.final_env()
+            self.prewarmed_child = fast_data_types.get_boss().prewarm(slave, self.argv, self.cwd, fe, stdin)
+            pid = self.prewarmed_child.child_process_pid
+        else:
+            pid = fast_data_types.spawn(
+                self.final_exe, self.cwd, tuple(argv), env, master, slave, stdin_read_fd, stdin_write_fd,
+                ready_read_fd, ready_write_fd, tuple(handled_signals))
         os.close(slave)
         self.pid = pid
         self.child_fd = master
-        if stdin is not None:
-            os.close(stdin_read_fd)
-            fast_data_types.thread_write(stdin_write_fd, stdin)
-        os.close(ready_read_fd)
-        self.terminal_ready_fd = ready_write_fd
+        if not self.is_prewarmed:
+            if stdin is not None:
+                os.close(stdin_read_fd)
+                fast_data_types.thread_write(stdin_write_fd, stdin)
+            os.close(ready_read_fd)
+            self.terminal_ready_fd = ready_write_fd
         if self.child_fd is not None:
-            remove_blocking(self.child_fd)
+            os.set_blocking(self.child_fd, False)
         return pid
 
-    def mark_terminal_ready(self) -> None:
-        os.close(self.terminal_ready_fd)
+    def __del__(self) -> None:
+        fd = getattr(self, 'terminal_ready_fd', -1)
+        if fd > -1:
+            os.close(fd)
         self.terminal_ready_fd = -1
 
+    def mark_terminal_ready(self) -> None:
+        if self.is_prewarmed:
+            fast_data_types.get_boss().prewarm.mark_child_as_ready(self.prewarmed_child.child_id)
+        else:
+            os.close(self.terminal_ready_fd)
+            self.terminal_ready_fd = -1
+
     def cmdline_of_pid(self, pid: int) -> List[str]:
         try:
             ans = cmdline_of_pid(pid)
         except Exception:
             ans = []
-        if not ans and pid == self.pid:
+        if pid == self.pid and (not ans or (self.is_prewarmed and ans == cmdline_of_prewarmer())):
             ans = list(self.argv)
         return ans
 
@@ -411,3 +458,24 @@ class Child:
             with suppress(Exception):
                 return environ_of_process(pid)
         return {}
+
+    def send_signal_for_key(self, key_num: int) -> bool:
+        import signal
+        import termios
+        if self.child_fd is None:
+            return False
+        t = termios.tcgetattr(self.child_fd)
+        if not t[3] & termios.ISIG:
+            return False
+        cc = t[-1]
+        if key_num == cc[termios.VINTR]:
+            s = signal.SIGINT
+        elif key_num == cc[termios.VSUSP]:
+            s = signal.SIGTSTP
+        elif key_num == cc[termios.VQUIT]:
+            s = signal.SIGQUIT
+        else:
+            return False
+        pgrp = os.tcgetpgrp(self.child_fd)
+        os.killpg(pgrp, s)
+        return True
