@@ -1,22 +1,24 @@
 #!/usr/bin/env python3
-# vim:fileencoding=utf-8
 # License: GPL v3 Copyright: 2017, Kovid Goyal <kovid at kovidgoyal.net>
 
-import os
 import re
+import shlex
 import sys
 from collections import deque
 from typing import (
-    Any, Callable, Dict, FrozenSet, Iterable, Iterator, List, Match, Optional,
-    Sequence, Set, Tuple, Type, TypeVar, Union, cast
+    Any, Callable, Dict, FrozenSet, Iterator, List, Match, Optional, Sequence,
+    Tuple, Type, TypeVar, Union, cast
 )
 
 from .cli_stub import CLIOptions
 from .conf.utils import resolve_config
-from .config import KeyAction
-from .constants import appname, defconf, is_macos, is_wayland, str_version
-from .options_stub import Options as OptionsStub
-from .typing import BadLineType, KeySpec, SequenceMap, TypedDict
+from .constants import (
+    appname, clear_handled_signals, defconf, is_macos, str_version, website_url
+)
+from .fast_data_types import wcswidth
+from .options.types import Options as KittyOpts
+from .types import run_once
+from .typing import BadLineType, TypedDict
 
 
 class OptionDict(TypedDict):
@@ -27,102 +29,190 @@ class OptionDict(TypedDict):
     type: str
     default: Optional[str]
     condition: bool
+    completion: Dict[str, str]
 
 
 CONFIG_HELP = '''\
 Specify a path to the configuration file(s) to use. All configuration files are
-merged onto the builtin {conf_name}.conf, overriding the builtin values. This option
-can be specified multiple times to read multiple configuration files in
-sequence, which are merged. Use the special value NONE to not load a config
-file.
+merged onto the builtin :file:`{conf_name}.conf`, overriding the builtin values.
+This option can be specified multiple times to read multiple configuration files
+in sequence, which are merged. Use the special value :code:`NONE` to not load
+any config file.
 
 If this option is not specified, config files are searched for in the order:
-:file:`$XDG_CONFIG_HOME/{appname}/{conf_name}.conf`, :file:`~/.config/{appname}/{conf_name}.conf`, {macos_confpath}
-:file:`$XDG_CONFIG_DIRS/{appname}/{conf_name}.conf`. The first one that exists is used as the
-config file.
+:file:`$XDG_CONFIG_HOME/{appname}/{conf_name}.conf`,
+:file:`~/.config/{appname}/{conf_name}.conf`,{macos_confpath}
+:file:`$XDG_CONFIG_DIRS/{appname}/{conf_name}.conf`. The first one that exists
+is used as the config file.
 
-If the environment variable :env:`KITTY_CONFIG_DIRECTORY` is specified, that
+If the environment variable :envvar:`KITTY_CONFIG_DIRECTORY` is specified, that
 directory is always used and the above searching does not happen.
 
-If :file:`/etc/xdg/{appname}/{conf_name}.conf` exists it is merged before (i.e. with lower
-priority) than any user config files. It can be used to specify system-wide
-defaults for all users.
+If :file:`/etc/xdg/{appname}/{conf_name}.conf` exists, it is merged before (i.e.
+with lower priority) than any user config files. It can be used to specify
+system-wide defaults for all users. You can use either :code:`-` or
+:file:`/dev/stdin` to read the config from STDIN.
 '''.replace(
     '{macos_confpath}',
-    (':file:`~/Library/Preferences/{appname}/{conf_name}.conf`,' if is_macos else ''), 1
+    (' :file:`~/Library/Preferences/{appname}/{conf_name}.conf`,' if is_macos else ''), 1
 )
 
 
 def surround(x: str, start: int, end: int) -> str:
     if sys.stdout.isatty():
-        x = '\033[{}m{}\033[{}m'.format(start, x, end)
+        x = f'\033[{start}m{x}\033[{end}m'
     return x
 
 
+role_map: Dict[str, Callable[[str], str]] = {}
+
+
+def role(func: Callable[[str], str]) -> Callable[[str], str]:
+    role_map[func.__name__] = func
+    return func
+
+
+@role
 def emph(x: str) -> str:
     return surround(x, 91, 39)
 
 
+@role
 def cyan(x: str) -> str:
     return surround(x, 96, 39)
 
 
+@role
 def green(x: str) -> str:
     return surround(x, 32, 39)
 
 
+@role
 def blue(x: str) -> str:
     return surround(x, 34, 39)
 
 
+@role
 def yellow(x: str) -> str:
     return surround(x, 93, 39)
 
 
+@role
 def italic(x: str) -> str:
     return surround(x, 3, 23)
 
 
+@role
 def bold(x: str) -> str:
     return surround(x, 1, 22)
 
 
+@role
 def title(x: str) -> str:
     return blue(bold(x))
 
 
+@role
 def opt(text: str) -> str:
     return text
 
 
+@role
 def option(x: str) -> str:
     idx = x.rfind('--')
     if idx < 0:
         idx = x.find('-')
     if idx > -1:
         x = x[idx:]
-    parts = map(bold, x.split())
-    return ' '.join(parts)
+    return bold(x.rstrip('>'))
 
 
+@role
 def code(x: str) -> str:
-    return x
+    return cyan(x)
 
 
+def text_and_target(x: str) -> Tuple[str, str]:
+    parts = x.split('<', 1)
+    return parts[0].strip(), parts[-1].rstrip('>')
+
+
+@role
+def term(x: str) -> str:
+    return ref_hyperlink(x, 'term-')
+
+
+@role
 def kbd(x: str) -> str:
     return x
 
 
+@role
 def env(x: str) -> str:
-    return italic(x)
+    return ref_hyperlink(x, 'envvar-')
 
 
+role_map['envvar'] = role_map['env']
+
+
+def hyperlink_for_url(url: str, text: str) -> str:
+    if sys.stdout.isatty():
+        return f'\x1b]8;;{url}\x1b\\\x1b[4:3;58:5:4m{text}\x1b[4:0;59m\x1b]8;;\x1b\\'
+    return text
+
+
+@role
 def file(x: str) -> str:
     return italic(x)
 
 
+@role
 def doc(x: str) -> str:
-    return f'https://sw.kovidgoyal.net/kitty/{x}.html'
+    t, q = text_and_target(x)
+    if t == q:
+        from .conf.types import ref_map
+        m = ref_map()['doc']
+        q = q.strip('/')
+        if q in m:
+            x = f'{m[q]} <{t}>'
+    return ref_hyperlink(x, 'doc-')
+
+
+@run_once
+def hostname() -> str:
+    import socket
+    return socket.gethostname() or 'localhost'
+
+
+def ref_hyperlink(x: str, prefix: str = '') -> str:
+    t, q = text_and_target(x)
+    url = f'kitty+doc://{hostname()}/#ref={prefix}{q}'
+    return hyperlink_for_url(url, t)
+
+
+@role
+def ref(x: str) -> str:
+    return ref_hyperlink(x)
+
+
+@role
+def ac(x: str) -> str:
+    return ref_hyperlink(x, 'action-')
+
+
+@role
+def iss(x: str) -> str:
+    return ref_hyperlink(x, 'issues-')
+
+
+@role
+def pull(x: str) -> str:
+    return ref_hyperlink(x, 'pull-')
+
+
+@role
+def disc(x: str) -> str:
+    return ref_hyperlink(x, 'discussions-')
 
 
 OptionSpecSeq = List[Union[str, OptionDict]]
@@ -135,15 +225,19 @@ def parse_option_spec(spec: Optional[str] = None) -> Tuple[OptionSpecSeq, Option
     state = NORMAL
     lines = spec.splitlines()
     prev_line = ''
+    prev_indent = 0
     seq: OptionSpecSeq = []
     disabled: OptionSpecSeq = []
     mpat = re.compile('([a-z]+)=(.+)')
     current_cmd: OptionDict = {
         'dest': '', 'aliases': frozenset(), 'help': '', 'choices': frozenset(),
-        'type': '', 'condition': False, 'default': None
+        'type': '', 'condition': False, 'default': None, 'completion': {},
     }
     empty_cmd = current_cmd
 
+    def indent_of_line(x: str) -> int:
+        return len(x) - len(x.lstrip())
+
     for line in lines:
         line = line.rstrip()
         if state is NORMAL:
@@ -157,11 +251,11 @@ def parse_option_spec(spec: Optional[str] = None) -> Tuple[OptionSpecSeq, Option
                 current_cmd = {
                     'dest': parts[0][2:].replace('-', '_'), 'aliases': frozenset(parts), 'help': '',
                     'choices': frozenset(), 'type': '',
-                    'default': None, 'condition': True
+                    'default': None, 'condition': True, 'completion': {}
                 }
                 state = METADATA
                 continue
-            raise ValueError('Invalid option spec, unexpected line: {}'.format(line))
+            raise ValueError(f'Invalid option spec, unexpected line: {line}')
         elif state is METADATA:
             m = mpat.match(line)
             if m is None:
@@ -180,11 +274,24 @@ def parse_option_spec(spec: Optional[str] = None) -> Tuple[OptionSpecSeq, Option
                         current_cmd['dest'] = v
                     elif k == 'condition':
                         current_cmd['condition'] = bool(eval(v))
+                    elif k == 'completion':
+                        cv = current_cmd['completion'] = {}
+                        for x in shlex.split(v):
+                            ck, vv = x.split(':', 1)
+                            cv[ck] = vv
         elif state is HELP:
             if line:
+                current_indent = indent_of_line(line)
+                if current_indent > 1:
+                    if prev_indent == 0:
+                        current_cmd['help'] += '\n'
+                    else:
+                        line = line.strip()
+                prev_indent = current_indent
                 spc = '' if current_cmd['help'].endswith('\n') else ' '
                 current_cmd['help'] += spc + line
             else:
+                prev_indent = 0
                 if prev_line:
                     current_cmd['help'] += '\n\n'
                 else:
@@ -199,11 +306,13 @@ def parse_option_spec(spec: Optional[str] = None) -> Tuple[OptionSpecSeq, Option
 
 
 def prettify(text: str) -> str:
-    role_map = globals()
 
-    def sub(m: Match) -> str:
+    def identity(x: str) -> str:
+        return x
+
+    def sub(m: 'Match[str]') -> str:
         role, text = m.group(1, 2)
-        return str(role_map[role](text))
+        return role_map.get(role, identity)(text)
 
     text = re.sub(r':([a-z]+):`([^`]+)`', sub, text)
     return text
@@ -217,41 +326,61 @@ def version(add_rev: bool = False) -> str:
     rev = ''
     from . import fast_data_types
     if add_rev and hasattr(fast_data_types, 'KITTY_VCS_REV'):
-        rev = ' ({})'.format(fast_data_types.KITTY_VCS_REV[:10])
+        rev = f' ({fast_data_types.KITTY_VCS_REV[:10]})'
     return '{} {}{} created by {}'.format(italic(appname), green(str_version), rev, title('Kovid Goyal'))
 
 
 def wrap(text: str, limit: int = 80) -> Iterator[str]:
-    NORMAL, IN_FORMAT = 'NORMAL', 'IN_FORMAT'
-    state = NORMAL
-    last_space_at = None
-    chars_in_line = 0
-    breaks = []
-    for i, ch in enumerate(text):
-        if state is IN_FORMAT:
-            if ch == 'm':
-                state = NORMAL
-            continue
-        if ch == '\033':
-            state = IN_FORMAT
-            continue
-        if ch == ' ':
-            last_space_at = i
-        if chars_in_line < limit:
-            chars_in_line += 1
-            continue
-        if last_space_at is not None:
-            breaks.append(last_space_at)
-            last_space_at = None
-            chars_in_line = i - breaks[-1]
+    if not text.strip():
+        yield ''
+        return
+    in_escape = 0
+    current_line: List[str] = []
+    escapes: List[str] = []
+    current_word: List[str] = []
+    current_line_length = 0
 
-    lines: List[str] = []
-    for b in reversed(breaks):
-        lines.append(text[b:].lstrip())
-        text = text[:b]
-    if text:
-        lines.append(text)
-    return reversed(lines)
+    def print_word(ch: str = '') -> Iterator[str]:
+        nonlocal current_word, current_line, escapes, current_line_length
+        cw = ''.join(current_word)
+        w = wcswidth(cw)
+        if current_line_length + w > limit:
+            yield ''.join(current_line)
+            current_line = []
+            current_line_length = 0
+            cw = cw.strip()
+            current_word = [cw]
+        if escapes:
+            current_line.append(''.join(escapes))
+            escapes = []
+        if current_word:
+            current_line.append(cw)
+            current_line_length += w
+            current_word = []
+        if ch:
+            current_word.append(ch)
+
+    for i, ch in enumerate(text):
+        if in_escape > 0:
+            if in_escape == 1 and ch in '[]':
+                in_escape = 2 if ch == '[' else 3
+            if (in_escape == 2 and ch == 'm') or (in_escape == 3 and ch == '\\' and text[i-1] == '\x1b'):
+                in_escape = 0
+            escapes.append(ch)
+            continue
+        if ch == '\x1b':
+            in_escape = 1
+            if current_word:
+                yield from print_word()
+            escapes.append(ch)
+            continue
+        if current_word and ch.isspace() and ch != '\xa0':
+            yield from print_word(ch)
+        else:
+            current_word.append(ch)
+    yield from print_word()
+    if current_line:
+        yield ''.join(current_line)
 
 
 def get_defaults_from_seq(seq: OptionSpecSeq) -> Dict[str, Any]:
@@ -263,11 +392,13 @@ def get_defaults_from_seq(seq: OptionSpecSeq) -> Dict[str, Any]:
 
 
 default_msg = ('''\
-Run the :italic:`{appname}` terminal emulator. You can also specify the :italic:`program`
-to run inside :italic:`{appname}` as normal arguments following the :italic:`options`.
-For example: {appname} sh -c "echo hello, world. Press ENTER to quit; read"
+Run the :italic:`{appname}` terminal emulator. You can also specify the
+:italic:`program` to run inside :italic:`{appname}` as normal arguments
+following the :italic:`options`.
+For example: {appname} --hold sh -c "echo hello, world"
 
-For comprehensive documentation for kitty, please see: https://sw.kovidgoyal.net/kitty/''').format(appname=appname)
+For comprehensive documentation for kitty, please see: {url}''').format(
+    appname=appname, url=website_url())
 
 
 class PrintHelpForSeq:
@@ -289,11 +420,8 @@ class PrintHelpForSeq:
                 leading_indent = indent
             j = '\n' + (' ' * indent)
             lines: List[str] = []
-            for l in text.splitlines():
-                if l:
-                    lines.extend(wrap(l, limit=linesz - indent))
-                else:
-                    lines.append('')
+            for ln in text.splitlines():
+                lines.extend(wrap(ln, limit=linesz - indent))
             a((' ' * leading_indent) + j.join(lines))
 
         usage = '[program-to-run ...]' if usage is None else usage
@@ -307,20 +435,21 @@ class PrintHelpForSeq:
             a('{}:'.format(title('Options')))
         for opt in seq:
             if isinstance(opt, str):
-                a('{}:'.format(title(opt)))
+                a(f'{title(opt)}:')
                 continue
             help_text = opt['help']
             if help_text == '!':
                 continue  # hidden option
-            a('  ' + ', '.join(map(green, sorted(opt['aliases']))))
+            a('  ' + ', '.join(map(green, sorted(opt['aliases'], reverse=True))))
+            defval = opt.get('default')
             if not opt.get('type', '').startswith('bool-'):
-                blocks[-1] += '={}'.format(italic(opt['dest'].upper()))
+                if defval:
+                    dt = '=[{}]'.format(italic(defval))
+                    blocks[-1] += dt
             if opt.get('help'):
-                defval = opt.get('default')
-                t = help_text.replace('%default', str(defval))
-                wa(prettify(t.strip()), indent=4)
-                if defval is not None:
-                    wa('Default: {}'.format(defval), indent=4)
+                t = help_text.replace('%default', str(defval)).strip()
+                t = t.replace('#placeholder_for_formatting#', '')
+                wa(prettify(t), indent=4)
                 if opt.get('choices'):
                     wa('Choices: {}'.format(', '.join(opt['choices'])), indent=4)
                 a('')
@@ -328,7 +457,7 @@ class PrintHelpForSeq:
         text = '\n'.join(blocks) + '\n\n' + version()
         if print_help_for_seq.allow_pager and sys.stdout.isatty():
             import subprocess
-            p = subprocess.Popen(['less', '-isRXF'], stdin=subprocess.PIPE)
+            p = subprocess.Popen(['less', '-isRXF'], stdin=subprocess.PIPE, preexec_fn=clear_handled_signals)
             try:
                 p.communicate(text.encode('utf-8'))
             except KeyboardInterrupt:
@@ -357,7 +486,7 @@ def seq_as_rst(
     a('.. highlight:: sh')
     a('.. code-block:: sh')
     a('')
-    a('  {} {}{}'.format(appname, optstring, usage))
+    a(f'  {appname} {optstring}{usage}')
     a('')
     message = message or default_msg
     a(prettify_rst(message))
@@ -382,12 +511,13 @@ def seq_as_rst(
         if opt.get('help'):
             defval = opt.get('default')
             t = help_text.replace('%default', str(defval)).strip()
+            t = t.replace('#placeholder_for_formatting#', '')
             a('')
             a(textwrap.indent(prettify_rst(t), ' ' * 4))
             if defval is not None:
-                a(textwrap.indent('Default: :code:`{}`'.format(defval), ' ' * 4))
+                a(textwrap.indent(f'Default: :code:`{defval}`', ' ' * 4))
             if opt.get('choices'):
-                a(textwrap.indent('Choices: :code:`{}`'.format(', '.join(sorted(opt['choices']))), ' ' * 4))
+                a(textwrap.indent('Choices: {}'.format(', '.join(f':code:`{c}`' for c in sorted(opt['choices']))), ' ' * 4))
             a('')
 
     text = '\n'.join(blocks)
@@ -396,7 +526,7 @@ def seq_as_rst(
 
 def as_type_stub(seq: OptionSpecSeq, disabled: OptionSpecSeq, class_name: str, extra_fields: Sequence[str] = ()) -> str:
     from itertools import chain
-    ans: List[str] = ['class {}:'.format(class_name)]
+    ans: List[str] = [f'class {class_name}:']
     for opt in chain(seq, disabled):
         if isinstance(opt, str):
             continue
@@ -409,14 +539,17 @@ def as_type_stub(seq: OptionSpecSeq, disabled: OptionSpecSeq, class_name: str, e
         elif otype == 'list':
             t = 'typing.Sequence[str]'
         elif otype in ('choice', 'choices'):
-            t = 'str'
+            if opt['choices']:
+                t = 'typing.Literal[{}]'.format(','.join(f'{x!r}' for x in opt['choices']))
+            else:
+                t = 'str'
         elif otype.startswith('bool-'):
             t = 'bool'
         else:
-            raise ValueError('Unknown CLI option type: {}'.format(otype))
-        ans.append('    {}: {}'.format(name, t))
+            raise ValueError(f'Unknown CLI option type: {otype}')
+        ans.append(f'    {name}: {t}')
     for x in extra_fields:
-        ans.append('    {}'.format(x))
+        ans.append(f'    {x}')
     return '\n'.join(ans) + '\n\n\n'
 
 
@@ -455,7 +588,7 @@ class Options:
     def opt_for_alias(self, alias: str) -> OptionDict:
         opt = self.alias_map.get(alias)
         if opt is None:
-            raise SystemExit('Unknown option: {}'.format(emph(alias)))
+            raise SystemExit(f'Unknown option: {emph(alias)}')
         return opt
 
     def needs_arg(self, alias: str) -> bool:
@@ -516,7 +649,7 @@ def parse_cmdline(oc: Options, disabled: OptionSpecSeq, ans: Any, args: Optional
                 needs_arg = oc.needs_arg(parts[0])
                 if not needs_arg:
                     if len(parts) != 1:
-                        raise SystemExit('The {} option does not accept arguments'.format(emph(parts[0])))
+                        raise SystemExit(f'The {emph(parts[0])} option does not accept arguments')
                     oc.process_arg(parts[0])
                     continue
                 if len(parts) == 1:
@@ -531,7 +664,7 @@ def parse_cmdline(oc: Options, disabled: OptionSpecSeq, ans: Any, args: Optional
             oc.process_arg(current_option, arg)
             current_option, state = None, NORMAL
     if state is EXPECTING_ARG:
-        raise SystemExit('An argument is required for the option: {}'.format(emph(arg)))
+        raise SystemExit(f'An argument is required for the option: {emph(arg)}')
 
     for key, val in oc.values_map.items():
         setattr(ans, key, val)
@@ -548,88 +681,101 @@ def options_spec() -> str:
 dest=cls
 default={appname}
 condition=not is_macos
-Set the class part of the :italic:`WM_CLASS` window property. On Wayland, it sets the app id.
+Set the class part of the :italic:`WM_CLASS` window property. On Wayland, it
+sets the app id.
 
 
 --name
 condition=not is_macos
-Set the name part of the :italic:`WM_CLASS` property (defaults to using the value from :option:`{appname} --class`)
+Set the name part of the :italic:`WM_CLASS` property. Defaults to using the
+value from :option:`{appname} --class`.
 
 
 --title -T
-Set the window title. This will override any title set by the program running inside kitty. So
-only use this if you are running a program that does not set titles. If combined
-with :option:`{appname} --session` the title will be used for all windows created by the
-session, that do not set their own titles.
+Set the OS window title. This will override any title set by the program running
+inside kitty, permanently fixing the OS window's title. So only use this if you
+are running a program that does not set titles.
 
 
 --config -c
 type=list
+completion=type:file ext:conf group:"Config files" kwds:none,NONE
 {config_help}
 
 
 --override -o
 type=list
 Override individual configuration options, can be specified multiple times.
-Syntax: :italic:`name=value`. For example: :option:`kitty -o` font_size=20
+Syntax: :italic:`name=value`. For example: :option:`{appname} -o` font_size=20
 
 
---directory -d
+--directory --working-directory -d
 default=.
-Change to the specified directory when launching
+completion=type:directory
+Change to the specified directory when launching.
 
 
 --detach
 type=bool-set
 condition=not is_macos
-Detach from the controlling terminal, if any
+Detach from the controlling terminal, if any.
 
 
 --session
-Path to a file containing the startup :italic:`session` (tabs, windows, layout, programs).
-Use - to read from STDIN. See the README file for details and an example.
+completion=type:file ext:session relative:conf group:"Session files"
+Path to a file containing the startup :italic:`session` (tabs, windows, layout,
+programs). Use - to read from STDIN. See the :file:`README` file for details and
+an example. Environment variables are expanded, relative paths are resolved relative
+to the kitty configuration directory.
 
 
 --hold
 type=bool-set
 Remain open after child process exits. Note that this only affects the first
-window. You can quit by either using the close window shortcut or :kbd:`Ctrl+d`.
+window. You can quit by either using the close window shortcut or pressing any
+key.
 
 
 --single-instance -1
 type=bool-set
-If specified only a single instance of :italic:`{appname}` will run. New invocations will
-instead create a new top-level window in the existing :italic:`{appname}` instance. This
-allows :italic:`{appname}` to share a single sprite cache on the GPU and also reduces
-startup time. You can also have separate groups of :italic:`{appname}` instances by using the
-:option:`kitty --instance-group` option
+If specified only a single instance of :italic:`{appname}` will run. New
+invocations will instead create a new top-level window in the existing
+:italic:`{appname}` instance. This allows :italic:`{appname}` to share a single
+sprite cache on the GPU and also reduces startup time. You can also have
+separate groups of :italic:`{appname}` instances by using the :option:`{appname}
+--instance-group` option.
 
 
 --instance-group
-Used in combination with the :option:`kitty --single-instance` option. All :italic:`{appname}` invocations
-with the same :option:`kitty --instance-group` will result in new windows being created
-in the first :italic:`{appname}` instance within that group
+Used in combination with the :option:`{appname} --single-instance` option. All
+:italic:`{appname}` invocations with the same :option:`{appname}
+--instance-group` will result in new windows being created in the first
+:italic:`{appname}` instance within that group.
 
 
 --wait-for-single-instance-window-close
 type=bool-set
-Normally, when using :option:`--single-instance`, :italic:`{appname}` will open a new window in an existing
-instance and quit immediately. With this option, it will not quit till the newly opened
-window is closed. Note that if no previous instance is found, then :italic:`{appname}` will wait anyway,
+Normally, when using :option:`{appname} --single-instance`, :italic:`{appname}`
+will open a new window in an existing instance and quit immediately. With this
+option, it will not quit till the newly opened window is closed. Note that if no
+previous instance is found, then :italic:`{appname}` will wait anyway,
 regardless of this option.
 
 
 --listen-on
-Tell kitty to listen on the specified address for control
-messages. For example, :option:`{appname} --listen-on`=unix:/tmp/mykitty or
-:option:`{appname} --listen-on`=tcp:localhost:12345. On Linux systems, you can
-also use abstract UNIX sockets, not associated with a file, like this:
-:option:`{appname} --listen-on`=unix:@mykitty.  To control kitty, you can send
-it commands with :italic:`kitty @` using the :option:`kitty @ --to` option to
-specify this address. This option will be ignored, unless you set
-:opt:`allow_remote_control` to yes in :file:`kitty.conf`. Note that if you run
-:italic:`kitty @` within a kitty window, there is no need to specify the :italic:`--to`
-option as it is read automatically from the environment.
+Listen on the specified socket address for control messages. For example,
+:option:`{appname} --listen-on`=unix:/tmp/mykitty or :option:`{appname}
+--listen-on`=tcp:localhost:12345. On Linux systems, you can also use abstract
+UNIX sockets, not associated with a file, like this: :option:`{appname}
+--listen-on`=unix:@mykitty. Environment variables are expanded and relative
+paths are resolved with respect to the temporary directory. To control kitty,
+you can send commands to it with :italic:`{appname} @` using the
+:option:`{appname} @ --to` option to specify this address. Note that if you run
+:italic:`{appname} @` within a kitty window, there is no need to specify the
+:option:`{appname} @ --to` option as it will automatically read from the
+environment. Note that this will be ignored unless :opt:`allow_remote_control`
+is set to either: :code:`yes`, :code:`socket` or :code:`socket-only`.  For UNIX
+sockets, this can also be specified in :file:`{conf_name}.conf`.
 
 
 --start-as
@@ -643,44 +789,48 @@ Control how the initial kitty window is created.
 
 --version -v
 type=bool-set
-The current {appname} version
+The current {appname} version.
 
 
 --dump-commands
 type=bool-set
-Output commands received from child process to stdout
+Output commands received from child process to STDOUT.
 
 
 --replay-commands
-Replay previously dumped commands. Specify the path to a dump file previously created by --dump-commands. You
+Replay previously dumped commands. Specify the path to a dump file previously
+created by :option:`{appname} --dump-commands`. You
 can open a new kitty window to replay the commands with::
-
-    kitty sh -c "kitty --replay-commands /path/to/dump/file; read"
+    {appname} sh -c "{appname} --replay-commands /path/to/dump/file; read"
 
 
 --dump-bytes
-Path to file in which to store the raw bytes received from the child process
+Path to file in which to store the raw bytes received from the child process.
 
 
---debug-gl
+--debug-rendering --debug-gl
 type=bool-set
-Debug OpenGL commands. This will cause all OpenGL calls to check for errors
-instead of ignoring them. Useful when debugging rendering problems
+Debug rendering commands. This will cause all OpenGL calls to check for errors
+instead of ignoring them. Also prints out miscellaneous debug information.
+Useful when debugging rendering problems.
 
 
---debug-keyboard
+--debug-input --debug-keyboard
+dest=debug_keyboard
 type=bool-set
-This option will cause kitty to print out key events as they are received
+Print out key and mouse events as they are received.
 
 
 --debug-font-fallback
 type=bool-set
-Print out information about the selection of fallback fonts for characters not present in the main font.
+Print out information about the selection of fallback fonts for characters not
+present in the main font.
 
 
---debug-config
-type=bool-set
-Print out information about the system and kitty configuration.
+--watcher
+completion=type:file ext:py relative:conf group:"Watcher files"
+This option is deprecated in favor of the :opt:`watcher` option in
+:file:`{conf_name}.conf` and should not be used.
 
 
 --execute -e
@@ -688,8 +838,8 @@ type=bool-set
 !
 '''
         setattr(options_spec, 'ans', OPTIONS.format(
-            appname=appname, config_help=CONFIG_HELP.format(appname=appname, conf_name=appname)
-
+            appname=appname, conf_name=appname,
+            config_help=CONFIG_HELP.format(appname=appname, conf_name=appname),
         ))
     ans: str = getattr(options_spec, 'ans')
     return ans
@@ -733,107 +883,25 @@ def parse_args(
     return ans, parse_cmdline(oc, disabled, ans, args=args)
 
 
-SYSTEM_CONF = '/etc/xdg/kitty/kitty.conf'
-ShortcutMap = Dict[Tuple[KeySpec, ...], KeyAction]
+SYSTEM_CONF = f'/etc/xdg/{appname}/{appname}.conf'
 
 
-def print_shortcut(key_sequence: Iterable[KeySpec], action: KeyAction) -> None:
-    if not getattr(print_shortcut, 'maps', None):
-        from kitty.keys import defines
-        v = vars(defines)
-        mmap = {m[len('GLFW_MOD_'):].lower(): x for m, x in v.items() if m.startswith('GLFW_MOD_')}
-        kmap = {k[len('GLFW_KEY_'):].lower(): x for k, x in v.items() if k.startswith('GLFW_KEY_')}
-        krmap = {v: k for k, v in kmap.items()}
-        setattr(print_shortcut, 'maps', (mmap, krmap))
-    mmap, krmap = getattr(print_shortcut, 'maps')
-    keys = []
-    for key_spec in key_sequence:
-        names = []
-        mods, is_native, key = key_spec
-        for name, val in mmap.items():
-            if mods & val:
-                names.append(name)
-        if key:
-            if is_native:
-                from .fast_data_types import GLFW_KEY_UNKNOWN, glfw_get_key_name
-                kn = glfw_get_key_name(GLFW_KEY_UNKNOWN, key) or 'Unknown key'
-                names.append(kn)
-            else:
-                names.append(krmap[key])
-        keys.append('+'.join(names))
-
-    print('\t', ' > '.join(keys), action)
+def default_config_paths(conf_paths: Sequence[str]) -> Tuple[str, ...]:
+    return tuple(resolve_config(SYSTEM_CONF, defconf, conf_paths))
 
 
-def print_shortcut_changes(defns: ShortcutMap, text: str, changes: Set[Tuple[KeySpec, ...]]) -> None:
-    if changes:
-        print(title(text))
-
-        for k in sorted(changes):
-            print_shortcut(k, defns[k])
-
-
-def compare_keymaps(final: ShortcutMap, initial: ShortcutMap) -> None:
-    added = set(final) - set(initial)
-    removed = set(initial) - set(final)
-    changed = {k for k in set(final) & set(initial) if final[k] != initial[k]}
-    print_shortcut_changes(final, 'Added shortcuts:', added)
-    print_shortcut_changes(initial, 'Removed shortcuts:', removed)
-    print_shortcut_changes(final, 'Changed shortcuts:', changed)
-
-
-def flatten_sequence_map(m: SequenceMap) -> ShortcutMap:
-    ans: Dict[Tuple[KeySpec, ...], KeyAction] = {}
-    for key_spec, rest_map in m.items():
-        for r, action in rest_map.items():
-            ans[(key_spec,) + (r)] = action
-    return ans
-
-
-def compare_opts(opts: OptionsStub) -> None:
-    from .config import defaults, load_config
-    print('\nConfig options different from defaults:')
-    default_opts = load_config()
-    changed_opts = [
-        f for f in sorted(defaults._fields)  # type: ignore
-        if f not in ('key_definitions', 'keymap', 'sequence_map') and getattr(opts, f) != getattr(defaults, f)
-    ]
-    field_len = max(map(len, changed_opts)) if changed_opts else 20
-    fmt = '{{:{:d}s}}'.format(field_len)
-    for f in changed_opts:
-        print(title(fmt.format(f)), getattr(opts, f))
-
-    final_, initial_ = opts.keymap, default_opts.keymap
-    final: ShortcutMap = {(k,): v for k, v in final_.items()}
-    initial: ShortcutMap = {(k,): v for k, v in initial_.items()}
-    final_s, initial_s = map(flatten_sequence_map, (opts.sequence_map, default_opts.sequence_map))
-    final.update(final_s)
-    initial.update(initial_s)
-    compare_keymaps(final, initial)
-
-
-def create_opts(args: CLIOptions, debug_config: bool = False, accumulate_bad_lines: Optional[List[BadLineType]] = None) -> OptionsStub:
+def create_opts(args: CLIOptions, accumulate_bad_lines: Optional[List[BadLineType]] = None) -> KittyOpts:
     from .config import load_config
-    config = tuple(resolve_config(SYSTEM_CONF, defconf, args.config))
-    if debug_config:
-        print(version(add_rev=True))
-        print(' '.join(os.uname()))
-        if is_macos:
-            import subprocess
-            print(' '.join(subprocess.check_output(['sw_vers']).decode('utf-8').splitlines()).strip())
-        if os.path.exists('/etc/issue'):
-            with open('/etc/issue', encoding='utf-8', errors='replace') as f:
-                print(f.read().strip())
-        if os.path.exists('/etc/lsb-release'):
-            with open('/etc/lsb-release', encoding='utf-8', errors='replace') as f:
-                print(f.read().strip())
-        config = tuple(x for x in config if os.path.exists(x))
-        if config:
-            print(green('Loaded config files:'), ', '.join(config))
-    overrides = (a.replace('=', ' ', 1) for a in args.override or ())
+    config = default_config_paths(args.config)
+    # Does not cover the case where `name =` when `=` is the value.
+    pat = re.compile(r'^([a-zA-Z0-9_]+)[ \t]*=')
+    overrides = (pat.sub(r'\1 ', a.lstrip()) for a in args.override or ())
     opts = load_config(*config, overrides=overrides, accumulate_bad_lines=accumulate_bad_lines)
-    if debug_config:
-        if not is_macos:
-            print('Running under:', green('Wayland' if is_wayland(opts) else 'X11'))
-        compare_opts(opts)
+    return opts
+
+
+def create_default_opts() -> KittyOpts:
+    from .config import load_config
+    config = default_config_paths(())
+    opts = load_config(*config)
     return opts
