@@ -6,7 +6,7 @@ import os
 import pwd
 import sys
 from contextlib import suppress
-from typing import TYPE_CHECKING, Iterable, NamedTuple, Optional, Set
+from typing import TYPE_CHECKING, Any, Iterable, NamedTuple, Optional, Set
 
 from .types import run_once
 
@@ -22,14 +22,15 @@ class Version(NamedTuple):
 
 appname: str = 'kitty'
 kitty_face = '🐱'
-version: Version = Version(0, 25, 2)
+version: Version = Version(0, 26, 1)
 str_version: str = '.'.join(map(str, version))
 _plat = sys.platform.lower()
 is_macos: bool = 'darwin' in _plat
 is_freebsd: bool = 'freebsd' in _plat
 is_running_from_develop: bool = False
+RC_ENCRYPTION_PROTOCOL_VERSION = '1'
 if getattr(sys, 'frozen', False):
-    extensions_dir: str = getattr(sys, 'kitty_extensions_dir')
+    extensions_dir: str = getattr(sys, 'kitty_run_data')['extensions_dir']
 
     def get_frozen_base() -> str:
         global is_running_from_develop
@@ -61,7 +62,7 @@ else:
 
 @run_once
 def kitty_exe() -> str:
-    rpath = sys._xoptions.get('bundle_exe_dir')
+    rpath = getattr(sys, 'kitty_run_data').get('bundle_exe_dir')
     if not rpath:
         items = os.environ.get('PATH', '').split(os.pathsep) + [os.path.join(kitty_base_dir, 'kitty', 'launcher')]
         seen: Set[str] = set()
@@ -158,7 +159,7 @@ def runtime_dir() -> str:
     return candidate
 
 
-def wakeup() -> None:
+def wakeup_io_loop() -> None:
     from .fast_data_types import get_boss
     b = get_boss()
     if b is not None:
@@ -229,9 +230,50 @@ def read_kitty_resource(name: str, package_name: str = 'kitty') -> bytes:
     return read_binary(package_name, name)
 
 
-def website_url(doc_name: str = '') -> str:
+def website_url(doc_name: str = '', website: str = 'https://sw.kovidgoyal.net/kitty/') -> str:
     if doc_name:
-        doc_name = doc_name.rstrip('/')
-        if doc_name:
-            doc_name += '/'
-    return f'https://sw.kovidgoyal.net/kitty/{doc_name}'
+        base, _, frag = doc_name.partition('#')
+        base = base.rstrip('/')
+        if base:
+            base += '/'
+        doc_name = base + (f'#{frag}' if frag else '')
+    return website + doc_name.lstrip('/')
+
+
+handled_signals: Set[int] = set()
+
+
+def clear_handled_signals(*a: Any) -> None:
+    if not handled_signals:
+        return
+    import signal
+    if hasattr(signal, 'pthread_sigmask'):
+        signal.pthread_sigmask(signal.SIG_UNBLOCK, handled_signals)
+    for s in handled_signals:
+        signal.signal(s, signal.SIG_DFL)
+
+
+@run_once
+def local_docs() -> str:
+    d = os.path.dirname
+    base = d(d(kitty_exe()))
+    from_source = getattr(sys, 'kitty_run_data').get('from_source')
+    if is_macos and from_source and '/kitty.app/Contents/' in kitty_exe():
+        base = d(d(d(base)))
+    subdir = os.path.join('doc', 'kitty', 'html')
+    linux_ans = os.path.join(base, 'share', subdir)
+    if getattr(sys, 'frozen', False):
+        if is_macos:
+            return os.path.join(d(d(d(extensions_dir))), subdir)
+        return linux_ans
+    if os.path.isdir(linux_ans):
+        return linux_ans
+    if from_source:
+        sq = os.path.join(d(base), 'docs', '_build', 'html')
+        if os.path.isdir(sq):
+            return sq
+    for candidate in ('/usr', '/usr/local', '/opt/homebrew'):
+        q = os.path.join(candidate, 'share', subdir)
+        if os.path.isdir(q):
+            return q
+    return ''
