@@ -25,6 +25,7 @@ extern void cocoa_set_titlebar_appearance(void *w, unsigned int theme);
 extern void cocoa_set_titlebar_color(void *w, color_type color);
 extern bool cocoa_alt_option_key_pressed(unsigned long);
 extern void cocoa_toggle_secure_keyboard_entry(void);
+extern void cocoa_set_uncaught_exception_handler(void);
 extern void cocoa_update_menu_bar_title(PyObject*);
 extern size_t cocoa_get_workspace_ids(void *w, size_t *workspace_ids, size_t array_sz);
 extern monotonic_t cocoa_cursor_blink_interval(void);
@@ -108,6 +109,7 @@ update_os_window_viewport(OSWindow *window, bool notify_boss) {
         return; // no change, ignore
     }
     int min_width, min_height; min_size_for_os_window(window, &min_width, &min_height);
+    window->viewport_resized_at = monotonic();
     if (w <= 0 || h <= 0 || fw < min_width || fh < min_height || fw < w || fh < h) {
         log_error("Invalid geometry ignored: framebuffer: %dx%d window: %dx%d\n", fw, fh, w, h);
         if (!window->viewport_updated_at_least_once) {
@@ -463,6 +465,18 @@ application_close_requested_callback(int flags) {
     }
 }
 
+static char*
+get_current_selection(void) {
+    if (!global_state.boss) return NULL;
+    PyObject *ret = PyObject_CallMethod(global_state.boss, "get_active_selection", NULL);
+    if (!ret) { PyErr_Print(); return NULL; }
+    char* ans = NULL;
+    if (PyUnicode_Check(ret)) ans = strdup(PyUnicode_AsUTF8(ret));
+    Py_DECREF(ret);
+    return ans;
+}
+
+
 static void get_window_dpi(GLFWwindow *w, double *x, double *y);
 
 #ifdef __APPLE__
@@ -763,6 +777,7 @@ create_os_window(PyObject UNUSED *self, PyObject *args, PyObject *kw) {
         glfwWindowHint(GLFW_STENCIL_BITS, 0);
         if (OPT(hide_window_decorations) & 1) glfwWindowHint(GLFW_DECORATED, false);
         glfwSetApplicationCloseCallback(application_close_requested_callback);
+        glfwSetCurrentSelectionCallback(get_current_selection);
 #ifdef __APPLE__
         cocoa_set_activation_policy(OPT(macos_hide_from_tasks));
         glfwWindowHint(GLFW_COCOA_GRAPHICS_SWITCHING, true);
@@ -771,11 +786,12 @@ create_os_window(PyObject UNUSED *self, PyObject *args, PyObject *kw) {
 #endif
     }
 
-#ifndef __APPLE__
+#ifdef __APPLE__
+    glfwWindowHint(GLFW_COCOA_COLOR_SPACE, OPT(macos_colorspace));
+#else
     glfwWindowHintString(GLFW_X11_INSTANCE_NAME, wm_class_name);
     glfwWindowHintString(GLFW_X11_CLASS_NAME, wm_class_class);
     glfwWindowHintString(GLFW_WAYLAND_APP_ID, wm_class_class);
-    glfwWindowHint(GLFW_COCOA_COLOR_SPACE, OPT(macos_colorspace));
 #endif
 
     if (global_state.num_os_windows >= MAX_CHILDREN) {
@@ -810,12 +826,6 @@ create_os_window(PyObject UNUSED *self, PyObject *args, PyObject *kw) {
     if (ret == NULL) return NULL;
     int width = PyLong_AsLong(PyTuple_GET_ITEM(ret, 0)), height = PyLong_AsLong(PyTuple_GET_ITEM(ret, 1));
     Py_CLEAR(ret);
-    // The GLFW Wayland backend cannot create and show windows separately so we
-    // cannot call the pre_show_callback. See
-    // https://github.com/glfw/glfw/issues/1268 It doesn't matter since there
-    // is no startup notification in Wayland anyway. It amazes me that anyone
-    // uses Wayland as anything other than a butt for jokes.
-    if (global_state.is_wayland) glfwWindowHint(GLFW_VISIBLE, true);
     GLFWwindow *glfw_window = glfwCreateWindow(width, height, title, NULL, temp_window ? temp_window : common_context);
     if (temp_window) { glfwDestroyWindow(temp_window); temp_window = NULL; }
     if (glfw_window == NULL) { PyErr_SetString(PyExc_ValueError, "Failed to create GLFWwindow"); return NULL; }
@@ -830,25 +840,24 @@ create_os_window(PyObject UNUSED *self, PyObject *args, PyObject *kw) {
 #ifndef __APPLE__
     if (is_first_window) glfwSwapInterval(OPT(sync_to_monitor) && !global_state.is_wayland ? 1 : 0);
 #endif
-    glfwSwapBuffers(glfw_window);
+    // On Wayland the initial swap is allowed only after the first XDG configure event
+    if (glfwAreSwapsAllowed(glfw_window)) glfwSwapBuffers(glfw_window);
     glfwSetInputMode(glfw_window, GLFW_LOCK_KEY_MODS, true);
-    if (!global_state.is_wayland) {
-        PyObject *pret = PyObject_CallFunction(pre_show_callback, "N", native_window_handle(glfw_window));
-        if (pret == NULL) return NULL;
-        Py_DECREF(pret);
-        if (x != -1 && y != -1) glfwSetWindowPos(glfw_window, x, y);
-        glfwShowWindow(glfw_window);
+    PyObject *pret = PyObject_CallFunction(pre_show_callback, "N", native_window_handle(glfw_window));
+    if (pret == NULL) return NULL;
+    Py_DECREF(pret);
+    if (x != -1 && y != -1) glfwSetWindowPos(glfw_window, x, y);
+    glfwShowWindow(glfw_window);
 #ifdef __APPLE__
-        float n_xscale, n_yscale;
-        double n_xdpi, n_ydpi;
-        get_window_content_scale(glfw_window, &n_xscale, &n_yscale, &n_xdpi, &n_ydpi);
-        if (n_xdpi != xdpi || n_ydpi != ydpi) {
-            // this can happen if the window is moved by the OS to a different monitor when shown
-            xdpi = n_xdpi; ydpi = n_ydpi;
-            fonts_data = load_fonts_data(OPT(font_size), xdpi, ydpi);
-        }
-#endif
+    float n_xscale, n_yscale;
+    double n_xdpi, n_ydpi;
+    get_window_content_scale(glfw_window, &n_xscale, &n_yscale, &n_xdpi, &n_ydpi);
+    if (n_xdpi != xdpi || n_ydpi != ydpi) {
+        // this can happen if the window is moved by the OS to a different monitor when shown
+        xdpi = n_xdpi; ydpi = n_ydpi;
+        fonts_data = load_fonts_data(OPT(font_size), xdpi, ydpi);
     }
+#endif
     if (is_first_window) {
         PyObject *ret = PyObject_CallFunction(load_programs, "O", is_semi_transparent ? Py_True : Py_False);
         if (ret == NULL) return NULL;
@@ -1029,6 +1038,9 @@ glfw_init(PyObject UNUSED *self, PyObject *args) {
     const char* path;
     int debug_keyboard = 0, debug_rendering = 0;
     if (!PyArg_ParseTuple(args, "s|pp", &path, &debug_keyboard, &debug_rendering)) return NULL;
+#ifdef __APPLE__
+    cocoa_set_uncaught_exception_handler();
+#endif
     const char* err = load_glfw(path);
     if (err) { PyErr_SetString(PyExc_RuntimeError, err); return NULL; }
     glfwSetErrorCallback(error_callback);
@@ -1349,7 +1361,7 @@ is_mouse_hidden(OSWindow *w) {
 
 void
 swap_window_buffers(OSWindow *os_window) {
-    glfwSwapBuffers(os_window->handle);
+    if (glfwAreSwapsAllowed(os_window->handle)) glfwSwapBuffers(os_window->handle);
 }
 
 void
@@ -1362,7 +1374,8 @@ should_os_window_be_rendered(OSWindow* w) {
     return (
             glfwGetWindowAttrib(w->handle, GLFW_ICONIFIED) ||
             !glfwGetWindowAttrib(w->handle, GLFW_VISIBLE) ||
-            glfwGetWindowAttrib(w->handle, GLFW_OCCLUDED)
+            glfwGetWindowAttrib(w->handle, GLFW_OCCLUDED) ||
+            !glfwAreSwapsAllowed(w->handle)
        ) ? false : true;
 }
 
@@ -1629,11 +1642,12 @@ void cleanup_glfw(void) {
 #endif
 }
 
-// constants {{{
 bool
 init_glfw(PyObject *m) {
     if (PyModule_AddFunctions(m, module_methods) != 0) return false;
     register_at_exit_cleanup_func(GLFW_CLEANUP_FUNC, cleanup_glfw);
+
+// constants {{{
 #define ADDC(n) if(PyModule_AddIntConstant(m, #n, n) != 0) return false;
     ADDC(GLFW_RELEASE);
     ADDC(GLFW_PRESS);
@@ -1875,9 +1889,8 @@ init_glfw(PyObject *m) {
 // ---
     ADDC(GLFW_CONNECTED);
     ADDC(GLFW_DISCONNECTED);
-
-return true;
 #undef ADDC
+// }}}
+
+    return true;
 }
-// }}}
-// }}}
