@@ -157,9 +157,8 @@ on_key_input(GLFWkeyevent *ev) {
         case GLFW_IME_NONE:
             // for macOS, update ime position on every key input
             // because the position is required before next input
-#if defined(__APPLE__)
+            // On Linux this is needed by Fig integration: https://github.com/kovidgoyal/kitty/issues/5241
             update_ime_position(w, screen);
-#endif
             break;
         default:
             debug("invalid state, ignoring\n");
@@ -219,6 +218,9 @@ on_key_input(GLFWkeyevent *ev) {
         schedule_write_to_child(w->id, 1, text, strlen(text));
         debug("sent key as text to child\n");
     } else if (size > 0) {
+        if (size == 1 && screen->modes.mHANDLE_TERMIOS_SIGNALS) {
+            if (screen_send_signal_for_key(screen, *encoded_key)) return;
+        }
         schedule_write_to_child(w->id, 1, encoded_key, size);
         debug("sent encoded key to child\n");
     } else {
@@ -281,11 +283,200 @@ static PyMethodDef module_methods[] = {
     {0}
 };
 
+// SingleKey {{{
+typedef uint32_t keybitfield;
+#define KEY_BITS 21
+#define MOD_BITS 10
+#if 1 << (MOD_BITS-1) < GLFW_MOD_KITTY
+#error "Not enough mod bits"
+#endif
+typedef union Key {
+    struct {
+        keybitfield mods : MOD_BITS;
+        keybitfield is_native: 1;
+        keybitfield key : KEY_BITS;
+    };
+    keybitfield val;
+} Key;
+
+static PyTypeObject SingleKey_Type;
+static char *SingleKey_kwds[] = {"mods", "is_native", "key", NULL};
+typedef struct {
+    PyObject_HEAD
+
+    Key key;
+    bool defined_with_kitty_mod;
+} SingleKey;
+
+static inline void
+SingleKey_set_vals(SingleKey *self, long key, unsigned short mods, int is_native) {
+    if (key >= 0 && (unsigned long)key <= BIT_MASK(keybitfield, KEY_BITS)) {
+        keybitfield k = (keybitfield)(unsigned long)key;
+        self->key.key = k & BIT_MASK(keybitfield, KEY_BITS);
+    }
+    if (!(mods & 1 << (MOD_BITS + 1))) self->key.mods = mods & BIT_MASK(u_int32_t, MOD_BITS);
+    if (is_native > -1) self->key.is_native = is_native ? 1 : 0;
+}
+
+static PyObject *
+SingleKey_new(PyTypeObject *type, PyObject *args, PyObject *kw) {
+    long key = -1; unsigned short mods = 1 << (MOD_BITS + 1); int is_native = -1;
+    if (!PyArg_ParseTupleAndKeywords(args, kw, "|Hpl", SingleKey_kwds, &mods, &is_native, &key)) return NULL;
+    SingleKey *self = (SingleKey *)type->tp_alloc(type, 0);
+    if (self) SingleKey_set_vals(self, key, mods, is_native);
+    return (PyObject*)self;
+}
+
+static void
+SingleKey_dealloc(SingleKey* self) {
+    Py_TYPE(self)->tp_free((PyObject*)self);
+}
+
+static PyObject*
+SingleKey_repr(PyObject *s) {
+    SingleKey *self = (SingleKey*)s;
+    char buf[128];
+    int pos = 0;
+    pos += PyOS_snprintf(buf + pos, sizeof(buf) - pos, "SingleKey(");
+    unsigned int mods = self->key.mods;
+    if (mods) pos += PyOS_snprintf(buf + pos, sizeof(buf) - pos, "mods=%u, ", mods);
+    if (self->key.is_native) pos += PyOS_snprintf(buf + pos, sizeof(buf) - pos, "is_native=True, ");
+    unsigned long key = self->key.key;
+    if (key) pos += PyOS_snprintf(buf + pos, sizeof(buf) - pos, "key=%lu, ", key);
+    if (buf[pos-1] == ' ') pos -= 2;
+    pos += PyOS_snprintf(buf + pos, sizeof(buf) - pos, ")");
+    return PyUnicode_FromString(buf);
+}
+
+static PyObject*
+SingleKey_get_key(SingleKey *self, void UNUSED *closure) {
+    const unsigned long val = self->key.key;
+    return PyLong_FromUnsignedLong(val);
+}
+
+static PyObject*
+SingleKey_get_mods(SingleKey *self, void UNUSED *closure) {
+    const unsigned long mods = self->key.mods;
+    return PyLong_FromUnsignedLong(mods);
+
+}
+
+static PyObject*
+SingleKey_get_is_native(SingleKey *self, void UNUSED *closure) {
+    if (self->key.is_native) Py_RETURN_TRUE;
+    Py_RETURN_FALSE;
+}
+
+static PyObject*
+SingleKey_defined_with_kitty_mod(SingleKey *self, void UNUSED *closure) {
+    if (self->defined_with_kitty_mod || (self->key.mods & GLFW_MOD_KITTY)) Py_RETURN_TRUE;
+    Py_RETURN_FALSE;
+}
+
+
+static PyGetSetDef SingleKey_getsetters[] = {
+    {"key", (getter)SingleKey_get_key, NULL, "The key as an integer", NULL},
+    {"mods", (getter)SingleKey_get_mods, NULL, "The modifiers as an integer", NULL},
+    {"is_native", (getter)SingleKey_get_is_native, NULL, "A bool", NULL},
+    {"defined_with_kitty_mod", (getter)SingleKey_defined_with_kitty_mod, NULL, "A bool", NULL},
+    {NULL}  /* Sentinel */
+};
+
+static Py_hash_t
+SingleKey_hash(PyObject *self) {
+    Py_hash_t ans = ((SingleKey*)self)->key.val;
+    if (ans == -1) ans = -2;
+    return ans;
+}
+
+static PyObject*
+SingleKey_richcompare(PyObject *self, PyObject *other, int op) {
+    if (!PyObject_TypeCheck(other, &SingleKey_Type)) { PyErr_SetString(PyExc_TypeError, "Cannot compare SingleKey to other objects"); return NULL; }
+    SingleKey *a = (SingleKey*)self, *b = (SingleKey*)other;
+    Py_RETURN_RICHCOMPARE(a->key.val, b->key.val, op);
+}
+
+static Py_ssize_t
+SingleKey___len__(PyObject *self UNUSED) {
+    return 3;
+}
+
+static PyObject *
+SingleKey_item(PyObject *o, Py_ssize_t i) {
+    SingleKey *self = (SingleKey*)o;
+    switch(i) {
+        case 0:
+            return SingleKey_get_mods(self, NULL);
+        case 1:
+            return SingleKey_get_is_native(self, NULL);
+        case 2:
+            return SingleKey_get_key(self, NULL);
+    }
+    PyErr_SetString(PyExc_IndexError, "tuple index out of range");
+    return NULL;
+}
+
+static PySequenceMethods SingleKey_sequence_methods = {
+    .sq_length = SingleKey___len__,
+    .sq_item = SingleKey_item,
+};
+
+static PyObject*
+SingleKey_resolve_kitty_mod(SingleKey *self, PyObject *km) {
+    if (!(self->key.mods & GLFW_MOD_KITTY)) { Py_INCREF(self); return (PyObject*)self; }
+    unsigned long kitty_mod = PyLong_AsUnsignedLong(km);
+    if (PyErr_Occurred()) return NULL;
+    SingleKey *ans = (SingleKey*)SingleKey_Type.tp_alloc(&SingleKey_Type, 0);
+    if (!ans) return NULL;
+    ans->key.val = self->key.val;
+    ans->key.mods = (ans->key.mods & ~GLFW_MOD_KITTY) | kitty_mod;
+    ans->defined_with_kitty_mod = true;
+    return (PyObject*)ans;
+}
+
+static PyObject*
+SingleKey_replace(SingleKey *self, PyObject *args, PyObject *kw) {
+    long key = -2; unsigned short mods = 1 << (MOD_BITS + 1); int is_native = -1;
+    if (!PyArg_ParseTupleAndKeywords(args, kw, "|Hpl", SingleKey_kwds, &mods, &is_native, &key)) return NULL;
+    SingleKey *ans = (SingleKey*)SingleKey_Type.tp_alloc(&SingleKey_Type, 0);
+    if (ans) {
+        if (key == -1) key = 0;
+        ans->key.val = self->key.val;
+        SingleKey_set_vals(ans, key, mods, is_native);
+    }
+    return (PyObject*)ans;
+}
+
+static PyMethodDef SingleKey_methods[] = {
+    {"_replace", (PyCFunction)(void (*) (void))SingleKey_replace, METH_VARARGS | METH_KEYWORDS, ""},
+    {"resolve_kitty_mod", (PyCFunction)SingleKey_resolve_kitty_mod, METH_O, ""},
+    {NULL}  /* Sentinel */
+};
+
+
+static PyTypeObject SingleKey_Type = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    .tp_name = "fast_data_types.SingleKey",
+    .tp_basicsize = sizeof(SingleKey),
+    .tp_dealloc = (destructor)SingleKey_dealloc,
+    .tp_flags = Py_TPFLAGS_DEFAULT,
+    .tp_doc = "Compact and fast representation of a single key as defined in the config",
+    .tp_new = SingleKey_new,
+    .tp_hash = SingleKey_hash,
+    .tp_richcompare = SingleKey_richcompare,
+    .tp_as_sequence = &SingleKey_sequence_methods,
+    .tp_repr = SingleKey_repr,
+    .tp_methods = SingleKey_methods,
+    .tp_getset = SingleKey_getsetters,
+}; // }}}
+
+
 bool
 init_keys(PyObject *module) {
     if (PyModule_AddFunctions(module, module_methods) != 0) return false;
     if (PyType_Ready(&PyKeyEvent_Type) < 0) return false;
     if (PyModule_AddObject(module, "KeyEvent", (PyObject *)&PyKeyEvent_Type) != 0) return 0;
     Py_INCREF(&PyKeyEvent_Type);
+    ADD_TYPE(SingleKey);
     return true;
 }
