@@ -57,13 +57,28 @@ of the active window in the tab is used as the tab title. The special value
 type=choices
 default=window
 choices=window,tab,os-window,overlay,background,clipboard,primary
-Where to launch the child process, in a new kitty :code:`window` in the current
-tab, a new :code:`tab`, or a new :code:`os-window` or an :code:`overlay` over
-the current window. Note that if the current window already has an overlay, then
-it will open a new window. The value of :code:`background` means the process
-will be run in the background. The values :code:`clipboard` and :code:`primary`
-are meant to work with :option:`--stdin-source <launch --stdin-source>` to copy
-data to the system clipboard or primary selection.
+Where to launch the child process:
+
+:code:`window`
+    A new :term:`kitty window <window>` in the current tab
+
+:code:`tab`
+    A new :term:`tab` in the current OS window
+
+:code:`os-window <os_window>`
+    A new :term:`operating system window <os_window>`
+
+:code:`overlay`
+    An :term:`overlay window <overlay>` covering the current active kitty window
+
+:code:`background`
+    The process will be run in the :italic:`background`, without a kitty window.
+
+:code:`clipboard`, :code:`primary`
+    These two are meant to work with :option:`--stdin-source <launch --stdin-source>` to copy
+    data to the :italic:`system clipboard` or :italic:`primary selection`.
+
+#placeholder_for_formatting#
 
 
 --keep-focus --dont-take-focus
@@ -73,6 +88,7 @@ opened window.
 
 
 --cwd
+completion=type:directory kwds:current,oldest,last_reported
 The working directory for the newly launched child. Use the special value
 :code:`current` to use the working directory of the currently active window.
 The special value :code:`last_reported` uses the last working directory reported
@@ -110,10 +126,9 @@ currently active window.
 --copy-env
 type=bool-set
 Copy the environment variables from the currently active window into the newly
-launched child process. Note that most shells only set environment variables
-for child processes, so this will only copy the environment variables that the
-shell process itself has, not the environment variables child processes inside
-the shell see. To copy that enviroment, use either the :ref:`clone-in-kitty
+launched child process. Note that this only copies the environment when the
+window was first created, as it is not possible to get updated environment variables
+from arbitrary processes. To copy that environment, use either the :ref:`clone-in-kitty
 <clone_shell>` feature or the kitty remote control feature with :option:`kitty
 @ launch --copy-env`.
 
@@ -141,6 +156,26 @@ enabled in :file:`kitty.conf`). Note that any program with the right level of
 permissions can still write to the pipes of any other program on the same
 computer and therefore can control kitty. It can, however, be useful to block
 programs running on other computers (for example, over SSH) or as other users.
+See :option:`--remote-control-password` for ways to restrict actions allowed by
+remote control.
+
+
+--remote-control-password
+type=list
+Restrict the actions remote control is allowed to take. This works like
+:opt:`remote_control_password`. You can specify a password and list of actions
+just as for :opt:`remote_control_password`. For example::
+    --remote-control-password '"my passphrase" get-* set-colors'
+
+This password will be in effect for this window only.
+Note that any passwords you have defined for :opt:`remote_control_password`
+in :file:`kitty.conf` are also in effect. You can override them by using the same password here.
+You can also disable all :opt:`remote_control_password` global passwords for this window, by using::
+    --remote-control-password '!'
+
+This option only takes effect if :option:`--allow-remote-control`
+is also specified. Can be specified multiple times to create multiple passwords.
+This option was added to kitty in version 0.26.0
 
 
 --stdin-source
@@ -148,17 +183,32 @@ type=choices
 default=none
 choices=none,@selection,@screen,@screen_scrollback,@alternate,@alternate_scrollback,@first_cmd_output_on_screen,@last_cmd_output,@last_visited_cmd_output
 Pass the screen contents as :file:`STDIN` to the child process.
-:code:`@selection` is the currently selected text. :code:`@screen` is the
-contents of the currently active window. :code:`@screen_scrollback` is the same
-as :code:`@screen`, but includes the scrollback buffer as well.
-:code:`@alternate` is the secondary screen of the current active window. For
-example if you run a full screen terminal application, the secondary screen will
-be the screen you return to when quitting the application.
-:code:`@first_cmd_output_on_screen` is the output from the first command run in
-the shell on screen, :code:`@last_cmd_output` is the output from the last
-command run in the shell, :code:`@last_visited_cmd_output` is the first output
-below the last scrolled position via :ac:`scroll_to_prompt`, this three needs
-:ref:`shell integration <shell_integration>` to work.
+
+:code:`@selection`
+    is the currently selected text.
+
+:code:`@screen`
+    is the contents of the currently active window.
+
+:code:`@screen_scrollback`
+    is the same as :code:`@screen`, but includes the scrollback buffer as well.
+
+:code:`@alternate`
+    is the secondary screen of the current active window. For example if you run
+    a full screen terminal application, the secondary screen will
+    be the screen you return to when quitting the application.
+
+:code:`@first_cmd_output_on_screen`
+    is the output from the first command run in the shell on screen.
+
+:code:`@last_cmd_output`
+    is the output from the last command run in the shell.
+
+:code:`@last_visited_cmd_output`
+    is the first output below the last scrolled position via :ac:`scroll_to_prompt`,
+    this needs :ref:`shell integration <shell_integration>` to work.
+
+#placeholder_for_formatting#
 
 
 --stdin-add-formatting
@@ -181,15 +231,15 @@ the same as for the :ac:`toggle_marker` action (see :doc:`/marks`).
 
 
 --os-window-class
-Set the *WM_CLASS* property on X11 and the application id property on Wayland
-for the newly created OS window when using :option:`--type=os-window
+Set the :italic:`WM_CLASS` property on X11 and the application id property on
+Wayland for the newly created OS window when using :option:`--type=os-window
 <launch --type>`. Defaults to whatever is used by the parent kitty process,
 which in turn defaults to :code:`kitty`.
 
 
 --os-window-name
-Set the *WM_NAME* property on X11 for the newly created OS Window when using
-:option:`--type=os-window <launch --type>`. Defaults to
+Set the :italic:`WM_NAME` property on X11 for the newly created OS Window when
+using :option:`--type=os-window <launch --type>`. Defaults to
 :option:`--os-window-class <launch --os-window-class>`.
 
 
@@ -200,8 +250,9 @@ use the title of the current OS window, if any.
 
 
 --logo
+completion=type:file ext:png group:"PNG images" relative:conf
 Path to a PNG image to use as the logo for the newly created window. See
-:opt:`window_logo_path`.
+:opt:`window_logo_path`. Relative paths are resolved from the kitty configuration directory.
 
 
 --logo-position
@@ -213,19 +264,20 @@ specified. See :opt:`window_logo_position`.
 type=float
 default=-1
 The amount the window logo should be faded into the background. Only takes
-effect if :option:`--logo` is specified. See :opt:`window_logo_position`.
+effect if :option:`--logo` is specified. See :opt:`window_logo_alpha`.
 
 
 --color
 type=list
 Change colors in the newly launched window. You can either specify a path to a
 :file:`.conf` file with the same syntax as :file:`kitty.conf` to read the colors
-from, or specify them individually, for example: ``--color background=white``
-``--color foreground=red``
+from, or specify them individually, for example: :code:`--color background=white
+--color foreground=red`.
 
 
 --watcher -w
 type=list
+completion=type:file ext:py relative:conf group:"Python scripts"
 Path to a Python file. Appropriately named functions in this file will be called
 for various events, such as when the window is resized, focused or closed. See
 the section on watchers in the launch command documentation: :ref:`watchers`.
@@ -317,6 +369,7 @@ def load_watch_modules(watchers: Iterable[str]) -> Optional[Watchers]:
 class LaunchKwds(TypedDict):
 
     allow_remote_control: bool
+    remote_control_passwords: Optional[Dict[str, Sequence[str]]]
     cwd_from: Optional[CwdRequest]
     cwd: Optional[str]
     location: Optional[str]
@@ -381,8 +434,16 @@ def launch(
         tm = boss.active_tab_manager
         opts.os_window_title = get_os_window_title(tm.os_window_id) if tm else None
     env = get_env(opts, active_child)
+    remote_control_restrictions: Optional[Dict[str, Sequence[str]]] = None
+    if opts.allow_remote_control and opts.remote_control_password:
+        from kitty.options.utils import remote_control_password
+        remote_control_restrictions = {}
+        for rcp in opts.remote_control_password:
+            for pw, rcp_items in remote_control_password(rcp, {}):
+                remote_control_restrictions[pw] = rcp_items
     kw: LaunchKwds = {
         'allow_remote_control': opts.allow_remote_control,
+        'remote_control_passwords': remote_control_restrictions,
         'cwd_from': None,
         'cwd': None,
         'location': None,
@@ -620,11 +681,10 @@ class EditCmd:
             self.file_localpath = os.path.join(self.tdir, self.file_name)
             with open(self.file_localpath, 'wb') as f:
                 f.write(self.file_data)
-        self.file_obj = open(self.file_localpath, 'rb')
         self.file_data = b''
         self.last_mod_time = self.file_mod_time
         if not self.opts.cwd:
-            self.opts.cwd = os.path.dirname(self.file_obj.name)
+            self.opts.cwd = os.path.dirname(self.file_localpath)
 
     def __del__(self) -> None:
         if self.tdir:
@@ -633,12 +693,12 @@ class EditCmd:
             self.tdir = ''
 
     def read_data(self) -> bytes:
-        self.file_obj.seek(0)
-        return self.file_obj.read()
+        with open(self.file_localpath, 'rb') as f:
+            return f.read()
 
     @property
     def file_mod_time(self) -> int:
-        return os.stat(self.file_obj.fileno()).st_mtime_ns
+        return os.stat(self.file_localpath).st_mtime_ns
 
     def schedule_check(self) -> None:
         if not self.abort_signaled:
@@ -733,7 +793,7 @@ def remote_edit(msg: str, window: Window) -> None:
         if q is not None:
             q.abort_signaled = c.abort_signaled
         return
-    cmdline = get_editor(path_to_edit=c.file_obj.name, line_number=c.line_number)
+    cmdline = get_editor(path_to_edit=c.file_localpath, line_number=c.line_number)
     w = launch(get_boss(), c.opts, cmdline, active=window)
     if w is not None:
         c.source_window_id = window.id
