@@ -10,18 +10,19 @@ from typing import Dict, Generator, List, Optional, Sequence, Tuple
 
 from .borders import load_borders_program
 from .boss import Boss
-from .child import set_default_env
+from .child import set_default_env, set_LANG_in_default_env
 from .cli import create_opts, parse_args
 from .cli_stub import CLIOptions
 from .conf.utils import BadLine
 from .config import cached_values_for
 from .constants import (
-    appname, beam_cursor_data_file, config_dir, glfw_path, is_macos,
-    is_wayland, kitty_exe, logo_png_file, running_in_kitty, website_url
+    appname, beam_cursor_data_file, clear_handled_signals, config_dir,
+    glfw_path, is_macos, is_wayland, kitty_exe, logo_png_file,
+    running_in_kitty, website_url
 )
 from .fast_data_types import (
-    GLFW_IBEAM_CURSOR, GLFW_MOD_ALT, GLFW_MOD_SHIFT, create_os_window,
-    free_font_data, glfw_init, glfw_terminate, load_png_data,
+    GLFW_IBEAM_CURSOR, GLFW_MOD_ALT, GLFW_MOD_SHIFT, SingleKey,
+    create_os_window, free_font_data, glfw_init, glfw_terminate, load_png_data,
     set_custom_cursor, set_default_window_icon, set_options
 )
 from .fonts.box_drawing import set_scale
@@ -29,8 +30,8 @@ from .fonts.render import set_font_family
 from .options.types import Options
 from .options.utils import DELETE_ENV_VAR
 from .os_window_size import initial_window_size_func
+from .prewarm import PrewarmProcess, fork_prewarm_process
 from .session import create_sessions, get_os_window_sizing_data
-from .types import SingleKey
 from .utils import (
     cleanup_ssh_control_masters, detach, expandvars, log_error,
     single_instance, startup_notification_handler, unix_socket_paths
@@ -140,7 +141,7 @@ def set_x11_window_icon() -> None:
     set_default_window_icon(f'{path}-128{ext}')
 
 
-def _run_app(opts: Options, args: CLIOptions, bad_lines: Sequence[BadLine] = ()) -> None:
+def _run_app(opts: Options, args: CLIOptions, prewarm: PrewarmProcess, bad_lines: Sequence[BadLine] = ()) -> None:
     global_shortcuts: Dict[str, SingleKey] = {}
     if is_macos:
         from collections import defaultdict
@@ -179,7 +180,7 @@ def _run_app(opts: Options, args: CLIOptions, bad_lines: Sequence[BadLine] = ())
                     pre_show_callback,
                     args.title or appname, args.name or args.cls or appname,
                     wincls, load_all_shaders, disallow_override_title=bool(args.title))
-        boss = Boss(opts, args, cached_values, global_shortcuts)
+        boss = Boss(opts, args, cached_values, global_shortcuts, prewarm)
         boss.start(window_id, startup_sessions)
         if bad_lines:
             boss.show_bad_config_lines(bad_lines)
@@ -196,12 +197,12 @@ class AppRunner:
         self.first_window_callback = lambda window_handle: None
         self.initial_window_size_func = initial_window_size_func
 
-    def __call__(self, opts: Options, args: CLIOptions, bad_lines: Sequence[BadLine] = ()) -> None:
+    def __call__(self, opts: Options, args: CLIOptions, prewarm: PrewarmProcess, bad_lines: Sequence[BadLine] = ()) -> None:
         set_scale(opts.box_drawing_scale)
         set_options(opts, is_wayland(), args.debug_rendering, args.debug_font_fallback)
         try:
             set_font_family(opts, debug_font_matching=args.debug_font_fallback)
-            _run_app(opts, args, bad_lines)
+            _run_app(opts, args, prewarm, bad_lines)
         finally:
             set_options(None)
             free_font_data()  # must free font data before glfw/freetype/fontconfig/opengl etc are finalized
@@ -228,6 +229,7 @@ def ensure_macos_locale() -> None:
                 else:
                     log_error(f'Could not set LANG Cocoa returns language as: {lang}')
             os.environ['LANG'] = f'{lang}.UTF-8'
+            set_LANG_in_default_env(os.environ['LANG'])
 
 
 @contextmanager
@@ -249,7 +251,7 @@ def setup_profiling() -> Generator[None, None, None]:
         with open(cg, 'wb') as f:
             subprocess.call(['pprof', '--callgrind', exe, '/tmp/kitty-profile.log'], stdout=f)
         try:
-            subprocess.Popen(['kcachegrind', cg])
+            subprocess.Popen(['kcachegrind', cg], preexec_fn=clear_handled_signals)
         except FileNotFoundError:
             subprocess.call(['pprof', '--text', exe, '/tmp/kitty-profile.log'])
             print('To view the graphical call data, use: kcachegrind', cg)
@@ -303,11 +305,12 @@ def prepend_if_not_present(path: str, paths_serialized: str) -> str:
 
 def ensure_kitty_in_path() -> None:
     # Ensure the correct kitty is in PATH
-    rpath = sys._xoptions.get('bundle_exe_dir')
+    krd = getattr(sys, 'kitty_run_data')
+    rpath = krd.get('bundle_exe_dir')
     if not rpath:
         return
     if rpath:
-        modify_path = is_macos or getattr(sys, 'frozen', False) or sys._xoptions.get('kitty_from_source') == '1'
+        modify_path = is_macos or getattr(sys, 'frozen', False) or krd.get('from_source')
         existing = shutil.which('kitty')
         if modify_path or not existing:
             env_path = os.environ.get('PATH', '')
@@ -316,14 +319,29 @@ def ensure_kitty_in_path() -> None:
                 os.environ['PATH'] = prepend_if_not_present(rpath, env_path)
 
 
+def setup_manpath(env: Dict[str, str]) -> None:
+    # Ensure kitty manpages are available in frozen builds
+    if not getattr(sys, 'frozen', False):
+        return
+    from .constants import local_docs
+    mp = os.environ.get('MANPATH', env.get('MANPATH', ''))
+    d = os.path.dirname
+    kitty_man = os.path.join(d(d(d(local_docs()))), 'man')
+    if not mp:
+        env['MANPATH'] = f'{kitty_man}:'
+    elif mp.startswith(':'):
+        env['MANPATH'] = f':{kitty_man}:{mp}'
+    else:
+        env['MANPATH'] = f'{kitty_man}:{mp}'
+
+
 def setup_environment(opts: Options, cli_opts: CLIOptions) -> None:
     from_config_file = False
     if not cli_opts.listen_on and opts.listen_on.startswith('unix:'):
         cli_opts.listen_on = opts.listen_on
         from_config_file = True
-    if cli_opts.listen_on and opts.allow_remote_control != 'n':
+    if cli_opts.listen_on:
         cli_opts.listen_on = expand_listen_on(cli_opts.listen_on, from_config_file)
-        os.environ['KITTY_LISTEN_ON'] = cli_opts.listen_on
     env = opts.env.copy()
     ensure_kitty_in_path()
     kitty_path = shutil.which('kitty')
@@ -333,6 +351,7 @@ def setup_environment(opts: Options, cli_opts: CLIOptions) -> None:
         # the other values mean the user doesn't want a PATH
         if child_path not in ('', DELETE_ENV_VAR) and child_path is not None:
             env['PATH'] = prepend_if_not_present(os.path.dirname(kitty_path), env['PATH'])
+    setup_manpath(env)
     set_default_env(env)
 
 
@@ -350,17 +369,11 @@ def set_locale() -> None:
             except Exception:
                 log_error('Failed to set locale with no LANG')
             os.environ['LANG'] = old_lang
+            set_LANG_in_default_env(old_lang)
 
 
 def _main() -> None:
     running_in_kitty(True)
-    with suppress(AttributeError):  # python compiled without threading
-        sys.setswitchinterval(1000.0)  # we have only a single python thread
-
-    try:
-        set_locale()
-    except Exception:
-        log_error('Failed to set locale, ignoring')
 
     args = sys.argv[1:]
     if is_macos and os.environ.pop('KITTY_LAUNCHED_BY_LAUNCH_SERVICES', None) == '1':
@@ -403,8 +416,20 @@ def _main() -> None:
             return
     bad_lines: List[BadLine] = []
     opts = create_opts(cli_opts, accumulate_bad_lines=bad_lines)
-    init_glfw(opts, cli_opts.debug_keyboard, cli_opts.debug_rendering)
     setup_environment(opts, cli_opts)
+    prewarm = fork_prewarm_process(opts)
+    if prewarm is None:
+        raise SystemExit(1)
+
+    # set_locale on macOS uses cocoa APIs when LANG is not set, so we have to
+    # call it after the fork
+    try:
+        set_locale()
+    except Exception:
+        log_error('Failed to set locale, ignoring')
+    with suppress(AttributeError):  # python compiled without threading
+        sys.setswitchinterval(1000.0)  # we have only a single python thread
+    init_glfw(opts, cli_opts.debug_keyboard, cli_opts.debug_rendering)
     if cli_opts.watcher:
         from .window import global_watchers
         global_watchers.set_extra(cli_opts.watcher)
@@ -412,7 +437,7 @@ def _main() -> None:
     try:
         with setup_profiling():
             # Avoid needing to launch threads to reap zombies
-            run_app(opts, cli_opts, bad_lines)
+            run_app(opts, cli_opts, prewarm, bad_lines)
     finally:
         glfw_terminate()
         cleanup_ssh_control_masters()
