@@ -21,6 +21,8 @@ typedef struct {
     size_t len;
 } UrlPrefix;
 
+typedef enum AdjustmentUnit { POINT = 0, PERCENT = 1, PIXEL = 2 } AdjustmentUnit;
+
 typedef struct {
     monotonic_t visual_bell_duration, cursor_blink_interval, cursor_stop_blinking_after, mouse_hide_wait, click_interval;
     double wheel_scroll_multiplier, touch_scroll_multiplier;
@@ -44,8 +46,6 @@ typedef struct {
     float macos_thicken_font;
     WindowTitleIn macos_show_window_title_in;
     char *bell_path;
-    int adjust_line_height_px, adjust_column_width_px, adjust_baseline_px;
-    float adjust_line_height_frac, adjust_column_width_frac, adjust_baseline_frac;
     float background_opacity, dim_opacity;
 
     char *background_image, *default_window_logo;
@@ -84,6 +84,9 @@ typedef struct {
     } tab_bar_margin_height;
     long macos_menubar_title_max_length;
     int macos_colorspace;
+    struct {
+        float val; AdjustmentUnit unit;
+    } underline_position, underline_thickness, strikethrough_position, strikethrough_thickness, cell_width, cell_height, baseline;
 } Options;
 
 typedef struct WindowLogoRenderData {
@@ -201,6 +204,7 @@ typedef struct {
     PyObject *window_title;
     bool disallow_title_changes, title_is_overriden;
     bool viewport_size_dirty, viewport_updated_at_least_once;
+    monotonic_t viewport_resized_at;
     LiveResizeInfo live_resize;
     bool has_pending_resizes, is_semi_transparent, shown_once, is_damaged;
     uint32_t offscreen_texture_id;
