@@ -10,8 +10,8 @@ from contextlib import suppress
 from operator import attrgetter
 from time import monotonic
 from typing import (
-    Any, Deque, Dict, Generator, Iterable, Iterator, List, NamedTuple,
-    Optional, Sequence, Set, Tuple, Union
+    Any, Deque, Dict, Generator, Iterable, Iterator, List,
+    NamedTuple, Optional, Sequence, Set, Tuple, Union
 )
 
 from .borders import Border, Borders
@@ -168,12 +168,16 @@ class Tab:  # {{{
         self.mark_tab_bar_dirty()
 
     def startup(self, session_tab: 'SessionTab') -> None:
-        for cmd in session_tab.windows:
-            if isinstance(cmd, SpecialWindowInstance):
-                self.new_special_window(cmd)
+        for window in session_tab.windows:
+            spec = window.launch_spec
+            if isinstance(spec, SpecialWindowInstance):
+                self.new_special_window(spec)
             else:
                 from .launch import launch
-                launch(get_boss(), cmd.opts, cmd.args, target_tab=self, force_target_tab=True)
+                launch(get_boss(), spec.opts, spec.args, target_tab=self, force_target_tab=True)
+            if window.resize_spec is not None:
+                self.resize_window(*window.resize_spec)
+
         self.windows.set_active_window_group_for(self.windows.all_windows[session_tab.active_window_idx])
 
     def serialize_state(self) -> Dict[str, Any]:
@@ -226,6 +230,10 @@ class Tab:  # {{{
                 ans += 1
         return ans
 
+    def get_cwd_of_active_window(self, oldest: bool = False) -> Optional[str]:
+        w = self.active_window
+        return w.get_cwd_of_child(oldest) if w else None
+
     def set_title(self, title: str) -> None:
         self.name = title or ''
         self.mark_tab_bar_dirty()
@@ -358,8 +366,8 @@ class Tab:  # {{{
         cwd_from: Optional[CwdRequest] = None,
         cwd: Optional[str] = None,
         env: Optional[Dict[str, str]] = None,
-        allow_remote_control: bool = False,
         is_clone_launch: str = '',
+        add_listen_on_env_var: bool = True,
     ) -> Child:
         check_for_suitability = True
         if cmd is None:
@@ -408,7 +416,7 @@ class Tab:  # {{{
         pwid = platform_window_id(self.os_window_id)
         if pwid is not None:
             fenv['WINDOWID'] = str(pwid)
-        ans = Child(cmd, cwd or self.cwd, stdin, fenv, cwd_from, allow_remote_control=allow_remote_control, is_clone_launch=is_clone_launch)
+        ans = Child(cmd, cwd or self.cwd, stdin, fenv, cwd_from, is_clone_launch=is_clone_launch, add_listen_on_env_var=add_listen_on_env_var)
         ans.fork()
         return ans
 
@@ -434,14 +442,16 @@ class Tab:  # {{{
         watchers: Optional[Watchers] = None,
         overlay_behind: bool = False,
         is_clone_launch: str = '',
+        remote_control_passwords: Optional[Dict[str, Sequence[str]]] = None,
     ) -> Window:
         child = self.launch_child(
-            use_shell=use_shell, cmd=cmd, stdin=stdin, cwd_from=cwd_from, cwd=cwd, env=env, allow_remote_control=allow_remote_control,
-            is_clone_launch=is_clone_launch
+            use_shell=use_shell, cmd=cmd, stdin=stdin, cwd_from=cwd_from, cwd=cwd, env=env,
+            is_clone_launch=is_clone_launch, add_listen_on_env_var=False if allow_remote_control and remote_control_passwords else True
         )
         window = Window(
             self, child, self.args, override_title=override_title,
-            copy_colors_from=copy_colors_from, watchers=watchers
+            copy_colors_from=copy_colors_from, watchers=watchers,
+            allow_remote_control=allow_remote_control, remote_control_passwords=remote_control_passwords
         )
         # Must add child before laying out so that resize_pty succeeds
         get_boss().add_child(window)
@@ -1080,7 +1090,7 @@ class TabManager:  # {{{
                 if w.has_activity_since_last_focus:
                     has_activity_since_last_focus = True
             ans.append(TabBarData(
-                title, t is at, needs_attention,
+                title, t is at, needs_attention, t.id,
                 len(t), t.num_window_groups, t.current_layout.name or '',
                 has_activity_since_last_focus, t.active_fg, t.active_bg,
                 t.inactive_fg, t.inactive_bg
