@@ -21,7 +21,9 @@ from typing import (
 from .child import ProcessDesc
 from .cli_stub import CLIOptions
 from .config import build_ansi_color_table
-from .constants import appname, config_dir, is_macos, wakeup
+from .constants import (
+    appname, clear_handled_signals, config_dir, is_macos, wakeup_io_loop
+)
 from .fast_data_types import (
     BGIMAGE_PROGRAM, BLIT_PROGRAM, CELL_BG_PROGRAM, CELL_FG_PROGRAM,
     CELL_PROGRAM, CELL_SPECIAL_PROGRAM, CURSOR_BEAM, CURSOR_BLOCK,
@@ -36,7 +38,7 @@ from .fast_data_types import (
     mouse_selection, move_cursor_to_mouse_if_in_prompt, pt_to_px,
     set_clipboard_string, set_titlebar_color, set_window_logo,
     set_window_padding, set_window_render_data, update_ime_position_for_window,
-    update_window_title, update_window_visibility
+    update_window_title, update_window_visibility, wakeup_main_loop
 )
 from .keys import keyboard_mode_name, mod_mask
 from .notify import NotificationCommand, handle_notification_cmd
@@ -46,8 +48,8 @@ from .terminfo import get_capabilities
 from .types import MouseEvent, WindowGeometry, ac, run_once
 from .typing import BossType, ChildType, EdgeLiteral, TabType, TypedDict
 from .utils import (
-    get_primary_selection, kitty_ansi_sanitizer_pat, load_shaders, log_error,
-    open_cmd, open_url, parse_color_set, path_from_osc7_url,
+    docs_url, get_primary_selection, kitty_ansi_sanitizer_pat, load_shaders,
+    log_error, open_cmd, open_url, parse_color_set, path_from_osc7_url,
     resolve_custom_file, resolved_shell, sanitize_title, set_primary_selection
 )
 
@@ -268,7 +270,7 @@ def as_text(
     if add_history:
         pht = pagerhist(screen, as_ansi, add_wrap_markers)
         h: List[str] = [pht] if pht else []
-        screen.historybuf.as_text(h.append, as_ansi, add_wrap_markers)
+        screen.as_text_for_history_buf(h.append, as_ansi, add_wrap_markers)
         if h:
             if not screen.linebuf.is_continued(0):
                 h[-1] += '\n'
@@ -459,6 +461,8 @@ global_watchers = GlobalWatchers()
 
 class Window:
 
+    window_custom_type: str = ''
+
     def __init__(
         self,
         tab: TabType,
@@ -466,7 +470,9 @@ class Window:
         args: CLIOptions,
         override_title: Optional[str] = None,
         copy_colors_from: Optional['Window'] = None,
-        watchers: Optional[Watchers] = None
+        watchers: Optional[Watchers] = None,
+        allow_remote_control: bool = False,
+        remote_control_passwords: Optional[Dict[str, Sequence[str]]] = None,
     ):
         if watchers:
             self.watchers = watchers
@@ -483,6 +489,7 @@ class Window:
         self.actions_on_focus_change: List[Callable[['Window', bool], None]] = []
         self.actions_on_removal: List[Callable[['Window'], None]] = []
         self.current_marker_spec: Optional[Tuple[str, Union[str, Tuple[Tuple[int, str], ...]]]] = None
+        self.kitten_result_processors: List[Callable[['Window', Any], None]] = []
         self.pty_resized_once = False
         self.last_reported_pty_size = (-1, -1, -1, -1)
         self.needs_attention = False
@@ -490,7 +497,6 @@ class Window:
         self.default_title = os.path.basename(child.argv[0] or appname)
         self.child_title = self.default_title
         self.title_stack: Deque[str] = deque(maxlen=10)
-        self.allow_remote_control = child.allow_remote_control
         self.id: int = add_window(tab.os_window_id, tab.id, self.title)
         self.margin = EdgeWidths()
         self.padding = EdgeWidths()
@@ -513,6 +519,27 @@ class Window:
             self.screen.copy_colors_from(copy_colors_from.screen)
         else:
             setup_colors(self.screen, opts)
+        self.remote_control_passwords = remote_control_passwords
+        self.allow_remote_control = allow_remote_control
+
+    def remote_control_allowed(self, pcmd: Dict[str, Any], extra_data: Dict[str, Any]) -> bool:
+        if not self.allow_remote_control:
+            return False
+        if not self.remote_control_passwords:
+            return True
+        pw = pcmd.get('password', '')
+        auth_items = self.remote_control_passwords.get(pw)
+        if pw == '!':
+            auth_items = None
+        if auth_items is None:
+            if '!' in self.remote_control_passwords:
+                raise PermissionError()
+            return False
+        from .remote_control import password_authorizer
+        pa = password_authorizer(auth_items)
+        if not pa.is_cmd_allowed(pcmd, self, False, extra_data):
+            raise PermissionError()
+        return True
 
     @property
     def file_transmission_control(self) -> 'FileTransmission':
@@ -598,7 +625,7 @@ class Window:
         )
 
     def serialize_state(self) -> Dict[str, Any]:
-        return {
+        ans = {
             'version': 1,
             'id': self.id,
             'child_title': self.child_title,
@@ -606,12 +633,16 @@ class Window:
             'default_title': self.default_title,
             'title_stack': list(self.title_stack),
             'allow_remote_control': self.allow_remote_control,
+            'remote_control_passwords': self.remote_control_passwords,
             'cwd': self.child.current_cwd or self.child.cwd,
             'env': self.child.environ,
             'cmdline': self.child.cmdline,
             'margin': self.margin.serialize(),
             'padding': self.padding.serialize(),
         }
+        if self.window_custom_type:
+            ans['window_custom_type'] = self.window_custom_type
+        return ans
 
     @property
     def current_colors(self) -> Dict[str, Optional[int]]:
@@ -690,7 +721,8 @@ class Window:
 
     def refresh(self) -> None:
         self.screen.mark_as_dirty()
-        wakeup()
+        wakeup_io_loop()
+        wakeup_main_loop()
 
     def set_geometry(self, new_geometry: WindowGeometry) -> None:
         if self.destroyed:
@@ -831,9 +863,10 @@ class Window:
             set_clipboard_string(url)
 
     def handle_remote_file(self, netloc: str, remote_path: str) -> None:
-        from .utils import SSHConnectionData
-        from kittens.ssh.main import get_connection_data
         from kittens.remote_file.main import is_ssh_kitten_sentinel
+        from kittens.ssh.main import get_connection_data
+
+        from .utils import SSHConnectionData
         args = self.ssh_kitten_cmdline()
         conn_data: Union[None, List[str], SSHConnectionData] = None
         if args:
@@ -852,6 +885,13 @@ class Window:
             '--ssh-connection-data', json.dumps(conn_data)
         )
 
+    def send_signal_for_key(self, key_num: int) -> bool:
+        try:
+            return self.child.send_signal_for_key(key_num)
+        except OSError as err:
+            log_error(f'Failed to send signal for key to child with err: {err}')
+            return False
+
     def focus_changed(self, focused: bool) -> None:
         if self.destroyed:
             return
@@ -903,7 +943,7 @@ class Window:
             import subprocess
             env = self.child.foreground_environ
             env['KITTY_CHILD_CMDLINE'] = ' '.join(map(shlex.quote, self.child.cmdline))
-            subprocess.Popen(cb, env=env, cwd=self.child.foreground_cwd)
+            subprocess.Popen(cb, env=env, cwd=self.child.foreground_cwd, preexec_fn=clear_handled_signals)
         if not self.is_active:
             changed = not self.needs_attention
             self.needs_attention = True
@@ -1024,6 +1064,15 @@ class Window:
     def handle_kitten_result(self, msg: str) -> None:
         import base64
         self.kitten_result = json.loads(base64.b85decode(msg))
+        for processor in self.kitten_result_processors:
+            try:
+                processor(self, self.kitten_result)
+            except Exception:
+                import traceback
+                traceback.print_exc()
+
+    def add_kitten_result_processor(self, callback: Callable[['Window', Any], None]) -> None:
+        self.kitten_result_processors.append(callback)
 
     def handle_overlay_ready(self, msg: str) -> None:
         boss = get_boss()
@@ -1295,6 +1344,7 @@ class Window:
     def destroy(self) -> None:
         self.call_watchers(self.watchers.on_close, {})
         self.destroyed = True
+        del self.kitten_result_processors
         if hasattr(self, 'screen'):
             if self.is_active and self.os_window_id == current_os_window():
                 # Cancel IME composition when window is destroyed
@@ -1615,11 +1665,25 @@ class Window:
 
         For example::
 
-            map F1 signal_child SIGTERM
+            map f1 signal_child SIGTERM
         ''')
     def signal_child(self, *signals: int) -> None:
         pid = self.child.pid_for_cwd
         if pid is not None:
             for sig in signals:
                 os.kill(pid, sig)
+
+    @ac('misc', '''
+    Display the specified kitty documentation, preferring a local copy, if found.
+
+    For example::
+
+        # show the config docs
+        map f1 show_kitty_doc conf
+        # show the ssh kitten docs
+        map f1 show_kitty_doc kittens/ssh
+    ''')
+    def show_kitty_doc(self, which: str = '') -> None:
+        url = docs_url(which)
+        get_boss().open_url(url)
     # }}}
