@@ -2,12 +2,14 @@
 # License: GPL v3 Copyright: 2016, Kovid Goyal <kovid at kovidgoyal.net>
 
 import os
+import sys
 import tempfile
 
 from kitty.config import build_ansi_color_table, defaults
 from kitty.fast_data_types import (
-    ColorProfile, Cursor as C, HistoryBuf, LineBuf, Color,
-    parse_input_from_terminal, truncate_point_for_length, wcswidth, wcwidth, strip_csi
+    Color, ColorProfile, Cursor as C, HistoryBuf, LineBuf,
+    parse_input_from_terminal, strip_csi, truncate_point_for_length, wcswidth,
+    wcwidth
 )
 from kitty.rgb import to_color
 from kitty.utils import is_path_in_temp_dir, sanitize_title
@@ -532,3 +534,23 @@ class TestDataTypes(BaseTest):
         q('a\x1bbc', 'ac')
         q('a\x1b[bc', 'ac')
         q('a\x1b[12;34:43mbc', 'abc')
+
+    def test_SingleKey(self):
+        from kitty.fast_data_types import (
+            GLFW_MOD_KITTY, GLFW_MOD_SHIFT, SingleKey
+        )
+        for m in (GLFW_MOD_KITTY, GLFW_MOD_SHIFT):
+            s = SingleKey(mods=m)
+            self.ae(s.mods, m)
+        self.ae(tuple(iter(SingleKey())), (0, False, 0))
+        self.ae(tuple(SingleKey(key=sys.maxunicode, mods=GLFW_MOD_SHIFT, is_native=True)), (GLFW_MOD_SHIFT, True, sys.maxunicode))
+        self.ae(repr(SingleKey()), 'SingleKey()')
+        self.ae(repr(SingleKey(key=23, mods=2, is_native=True)), 'SingleKey(mods=2, is_native=True, key=23)')
+        self.ae(repr(SingleKey(key=23, mods=2)), 'SingleKey(mods=2, key=23)')
+        self.ae(repr(SingleKey(key=23)), 'SingleKey(key=23)')
+        self.ae(repr(SingleKey(key=23)._replace(mods=2)), 'SingleKey(mods=2, key=23)')
+        self.ae(repr(SingleKey(key=23)._replace(key=-1, mods=GLFW_MOD_KITTY)), f'SingleKey(mods={GLFW_MOD_KITTY})')
+        self.assertEqual(SingleKey(key=1), SingleKey(key=1))
+        self.assertEqual(hash(SingleKey(key=1)), hash(SingleKey(key=1)))
+        self.assertNotEqual(hash(SingleKey(key=1, mods=2)), hash(SingleKey(key=1)))
+        self.assertNotEqual(SingleKey(key=1, mods=2), SingleKey(key=1))
