@@ -11,7 +11,7 @@ import unittest
 from contextlib import contextmanager
 from functools import lru_cache, partial
 
-from kitty.constants import kitty_base_dir, terminfo_dir
+from kitty.constants import kitty_base_dir, terminfo_dir, shell_integration_dir
 from kitty.fast_data_types import CURSOR_BEAM, CURSOR_BLOCK, CURSOR_UNDERLINE
 from kitty.shell_integration import (
     setup_bash_env, setup_fish_env, setup_zsh_env
@@ -40,6 +40,7 @@ def basic_shell_env(home_dir):
         'KITTY_SHELL_INTEGRATION': 'enabled',
         'KITTY_INSTALLATION_DIR': kitty_base_dir,
         'BASH_SILENCE_DEPRECATION_WARNING': '1',
+        'PYTHONDONTWRITEBYTECODE': '1',
     }
     for x in ('USER', 'LANG'):
         if os.environ.get(x):
@@ -173,10 +174,9 @@ function _set_status_prompt; function fish_prompt; echo -n "$pipestatus $status
             pty.wait_till(lambda: pty.screen_contents().count(right_prompt) == 1)
             self.ae(pty.screen_contents(), q)
 
-            # XDG_DATA_DIRS
-            pty.send_cmd_to_child('set -q XDG_DATA_DIRS; or echo ok')
-            pty.wait_till(lambda: pty.screen_contents().count(right_prompt) == 2)
-            self.ae(str(pty.screen.line(1)), 'ok')
+            # shell integration dir must no be in XDG_DATA_DIRS
+            pty.send_cmd_to_child(f'string match -q -- "*{shell_integration_dir}*" "$XDG_DATA_DIRS" || echo "XDD_OK"')
+            pty.wait_till(lambda: 'XDD_OK' in pty.screen_contents())
 
             # CWD reporting
             self.assertTrue(pty.screen.last_reported_cwd.endswith(self.home_dir))
@@ -188,6 +188,7 @@ function _set_status_prompt; function fish_prompt; echo -n "$pipestatus $status
             pty.wait_till(lambda: pty.screen.last_reported_cwd.endswith(self.home_dir))
 
             # completion and prompt marking
+            pty.wait_till(lambda: 'cd -' not in pty.screen_contents().splitlines()[-1])
             pty.send_cmd_to_child('clear')
             pty.wait_till(lambda: pty.screen_contents().count(right_prompt) == 1)
             pty.send_cmd_to_child('_test_comp_path')
@@ -285,6 +286,10 @@ PS1="{ps1}"
             pty.write_to_child('\x04')
             pty.send_cmd_to_child('clear')
             pty.wait_till(lambda: pty.callbacks.titlebuf)
+        with self.run_shell(shell='bash', rc=f'''PS1="{ps1}"\ndeclare LOCAL_KSI_VAR=1''') as pty:
+            pty.callbacks.clear()
+            pty.send_cmd_to_child('declare')
+            pty.wait_till(lambda: 'LOCAL_KSI_VAR' in pty.screen_contents())
         with self.run_shell(shell='bash', rc=f'''PS1="{ps1}"''') as pty:
             pty.callbacks.clear()
             pty.send_cmd_to_child('printf "%s\x16\a%s" "a" "b"')
