@@ -92,7 +92,7 @@ def run_man(args: Any) -> None:
 
 
 def run_html(args: Any) -> None:
-    call('make FAIL_WARN=1 "OPTS=-D analytics_id=UA-20736318-2" dirhtml', cwd=docs_dir)
+    call('make FAIL_WARN=1 "OPTS=-D analytics_id=G-XTJK3R7GF2" dirhtml', cwd=docs_dir)
     add_old_redirects('docs/_build/dirhtml')
 
 
@@ -450,11 +450,13 @@ def change_to_git_master() -> Generator[None, None, None]:
         branch_before = current_branch()
         if branch_before != 'master':
             subprocess.check_call(['git', 'switch', 'master'])
+            subprocess.check_call(['make', 'debug'])
         try:
             yield
         finally:
             if branch_before != 'master':
                 subprocess.check_call(['git', 'switch', branch_before])
+                subprocess.check_call(['make', 'debug'])
     finally:
         if stash_ref_before != safe_read('.git/refs/stash'):
             subprocess.check_call(['git', 'stash', 'pop'])
